/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Class NameValuePairList contains a list of name value pairs.
 */
public class NameValuePairList implements Iterable<NameValuePair>, Serializable {
	private static final long serialVersionUID = 1L;
	private List<NameValuePair> lst;
	/**
	 * Create instance of NameValuePairList. 
	 */
	public NameValuePairList() {
		lst = new ArrayList<NameValuePair>();
	}
	/**
	 * Store name value pair.
	 * @param name name
	 * @param value value
	 */
	public void put(String name, String value) {
		lst.add(new NameValuePair(name, value));
	}
	/**
	 * Get value.
	 * @param name name
	 * @return value
	 */
	public String get(String name) {
		for(NameValuePair nvp : lst) {
			if(nvp.getName().equals(name)) {
				return nvp.getValue();
			}
		}
		return null;
	}
	/**
	 * Get name.
	 * @param ix index
	 * @return name
	 */
	public String getName(int ix) {
		return lst.get(ix).getName();
	}
    /**
     * Get value.
     * @param ix index
     * @return value
     */
	public String getValue(int ix) {
		return lst.get(ix).getValue();
	}
	/**
	 * Get size.
	 * @return size
	 */
	public int size() {
		return lst.size();
	}
	/**
	 * Get iterator.
	 * @return iterator
	 */
	public Iterator<NameValuePair> iterator() {
		return lst.iterator();
	}
}
