/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model;

import java.util.List;

/**
 * Interface Persistable describes the functionality required by
 * classes to be persisted.
 */
public interface Persistable {
    /**
     * Get id identifying object.
     * @return id
     */
	public String getId();
	/**
	 * Get type (XML element name / database table name not Java class name).
	 * @return type
	 */
	public String getType();
	/**
	 * Get all single value fields.
	 * @return name value pair list
	 */
	public NameValuePairList getAttributes();
	/**
	 * Get all multi value fields.
	 * @return list of list of persistable objects
	 */
	public List<List<Persistable>> getChildren();
	/**
	 * Set all single value fields.
	 * @param attr value pair list
	 * @throws ModelException if problems
	 */
	public void setAttributes(NameValuePairList attr) throws ModelException;
	/**
	 * Set all multi value fields.
	 * @param childs list of list of persistable objects
	 * @throws ModelException if problems
	 */
	public void setChildren(List<List<Persistable>> childs) throws ModelException;
}
