/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model;

/**
 * Class PersistableBase is base class for classes that needs to
 * keep track of where they were loaded from so they can be saved
 * the same place.
 */
public abstract class PersistableBase extends Versioned {
	private static final long serialVersionUID = 1L;
	private String store;
	/**
	 * Create instance of PersistableBase.
	 * @param version
	 */
	public PersistableBase(Version version) { 
		super(version);
	}
	/**
	 * Get store.
	 * @return store
	 */
	public String getStore() {
		return store;
	}
	/**
	 * Set store.
	 * @param store store
	 */
	public void setStore(String store) {
		this.store = store;
	}
}
