/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model;

import java.awt.Component;

/**
 * Interface Persister describes the functionality required to
 * persist other classes.
 */
public interface Persister {
    /**
     * Get persister params.
     * @return persister params
     */
    public String getParams();
    /**
     * Set persister params.
     * @param params persister params
     */
    public void setParams(String params);
    /**
     * Save persistable object.
     * @param p persistable object
     * @param target location to save to
     * @param parent value for *
     * @throws ModelException if problems
     */
	public void saveSmart(SeparatePersistable p, String target, String parent) throws ModelException;
    /**
     * Save persistable object.
     * @param p persistable object (location to save to is stored in object)
     * @param parent value for *
     * @throws ModelException if problems
     */
	public void saveSmart(SeparatePersistable p, String parent) throws ModelException;
	/**
	 * Load persistable object.
	 * @param source location to load from
     * @param parent value for *
	 * @return persistable object
	 * @throws ModelException if problems
	 */
 	public SeparatePersistable loadSmart(String source, Class<? extends SeparatePersistable> clz, String parent) throws ModelException;
    /**
     * Save persistable object.
     * @param p persistable object
     * @param target location to save to
     * @throws ModelException if problems
     */
    public void save(SeparatePersistable p, String target) throws ModelException;
    /**
     * Save persistable object.
     * @param p persistable object (location to save to is stored in object)
     * @throws ModelException if problems
     */
    public void save(SeparatePersistable p) throws ModelException;
    /**
     * Load persistable object.
     * @param source location to load from
     * @return persistable object
     * @throws ModelException if problems
     */
    public SeparatePersistable load(String source, Class<? extends SeparatePersistable> clz) throws ModelException;
    /**
     * Prompt for load location.
     * @param gui Swing component
     * @param parloc parent location
     * @param typ type
     * @param def default
     * @return location
     */
 	public String promptLoad(Component gui, String parloc, String typ, String def);
    /**
     * Prompt for save location.
     * @param gui Swing component
     * @param parloc parent location
     * @param typ type
     * @param def default
     * @return location
     */
 	public String promptSave(Component gui, String parloc, String typ, String def);
}
