/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model;

import java.io.File;
import java.io.IOException;

/**
 * Class Util contains some static utility methods.
 */
public class Util {
    /**
     * Capitalize string.
     * @param s string
     * @return capitalized string
     */
	public static String toCap(String s) {
		return Character.toUpperCase(s.charAt(0)) + s.substring(1).toLowerCase();
	}
	/**
	 * Combine paths.
	 * @param path1 full filename
	 * @param path2 full filename
	 * @return filename with path1 directory and path2 name
	 */
    public static String combinePath(String path1, String path2) {
        String p1 = new File(path1).getParent();
        String p2 = new File(path2).getName();
        return p1 + System.getProperty("file.separator") + p2;     
    }
    /**
     * Check is filename is a valid input file.
     * @param fnm filename
     * @return true if valid false if not valid
     */
    public static boolean isValidInputFile(String fnm) {
        File f = new File(fnm);
        if(f.exists() && f.isFile()) {
            return true;
        } else {
            return false;
        }
    }
    /**
     * Check is filename is a valid output file.
     * @param fnm filename
     * @return true if valid false if not valid
     */
    public static boolean isValidOutputFile(String fnm) {
        File f = new File(fnm);
        try {
            f.getCanonicalPath();
            return true;
        } catch(IOException ex) {
            return false;
        }
    }
    /**
     * Convert double to string but treat it as int if it is.
     * @param x double value
     * @return string value
     */
	public static String intOrDoubleToString(double x) {
		if(Math.abs(x - Math.round(x)) < 0.001) {
			return Long.toString(Math.round(x));
		} else {
			return String.format("%.3f", x);
		}
	}

}
