/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model;

import java.io.Serializable;

/**
 * Class Version is base class for classes containing a version.
 */
public class Versioned implements Serializable, Cloneable {
	private static final long serialVersionUID = 1L;
	private Version version;
	/**
	 * Create instance of Versioned.
	 * @param version version
	 */
	public Versioned(Version version) {
		this.version = version;
	}
	/**
	 * Get version.
	 * @return version
	 */
	public Version getVersion() {
		return version;
	}
	/**
	 * Set version.
	 * @param version version
	 */
	public void setVersion(Version version) {
		this.version = version;
	}
	// cloneable
	public Versioned clone() {
	    return new Versioned(version);
	}
}
