/*
 * Copyright 2011 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.awt.Color;

/**
 * Class AppMetalThemeData contains simplified information about a theme.
 */
public class AppMetalThemeData {
    private Color border;
    private Color marked;
    private Color background;
    private static AppMetalThemeData instance = null;
    private AppMetalThemeData() {
        border = Color.DARK_GRAY;
        marked = new Color(0, 128, 224);
        background = Color.LIGHT_GRAY;
    }
    /**
     * Get single instance of theme.
     * @return theme
     */
    public static synchronized AppMetalThemeData getInstance() {
        if(instance == null) {
            instance = new AppMetalThemeData();
        }
        return instance;
    }
    /**
     * Get border color.
     * @return border color.
     */
    public Color getBorder() {
        return border;
    }
    /**
     * Set border color.
     * @param border border color
     */
    public void setBorder(Color border) {
        this.border = border;
    }
    /**
     * Get marked color.
     * @return marked color
     */
    public Color getMarked() {
        return marked;
    }
    /**
     * Set marked color.
     * @param marked marked color
     */
    public void setMarked(Color marked) {
        this.marked = marked;
    }
    /**
     * Get background color.
     * @return background color
     */
    public Color getBackground() {
        return background;
    }
    /**
     * Set background color.
     * @param background background color
     */
    public void setBackground(Color background) {
        this.background = background;
    }
}