/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.PaletteBase;
import dk.vajhoej.model.SeparatePersistable;
import dk.vajhoej.model.Util;
import dk.vajhoej.model.core.Box;
import dk.vajhoej.model.core.BoxPalette;
import dk.vajhoej.model.core.Component;
import dk.vajhoej.model.core.ComponentPalette;
import dk.vajhoej.model.core.Connection;
import dk.vajhoej.model.core.ConnectionPalette;
import dk.vajhoej.model.core.ExtModel;
import dk.vajhoej.model.core.Location;
import dk.vajhoej.model.core.Model;
import dk.vajhoej.model.graphic.adv.BoxGraphicWrapper;
import dk.vajhoej.model.graphic.adv.BoxPaletteGraphicWrapper;
import dk.vajhoej.model.graphic.adv.ComponentPaletteGraphicWrapper;
import dk.vajhoej.model.graphic.adv.ConnectionGraphicWrapper;
import dk.vajhoej.model.graphic.adv.ConnectionPaletteGraphicWrapper;
import dk.vajhoej.model.graphic.adv.FillerGraphic;
import dk.vajhoej.model.graphic.adv.GraphicEditPanel;
import dk.vajhoej.model.graphic.adv.HelpPanel;
import dk.vajhoej.model.graphic.adv.ModelGraphicWrapperAdv;
import dk.vajhoej.model.graphic.adv.MultiFilterPanel;
import dk.vajhoej.model.graphic.adv.ScriptPanel;
import dk.vajhoej.model.graphic.base.BaseGraphic;
import dk.vajhoej.model.graphic.base.SuperSquare;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;
import dk.vajhoej.model.graphic.config.GraphicConfig;
import dk.vajhoej.model.persist.XmlPersister;
import dk.vajhoej.model.version.OneNumberVersion;
import dk.vajhoej.model.version.StringVersion;
import dk.vajhoej.model.version.TwoNumberVersion;

/**
 * Class Application is the main GUI.
 */
public class Application extends JFrame {
	private static final long serialVersionUID = 1L;
	private static final String VERSION = "1.1";
	private Data data;
	private boolean main;
	private JComponent printDefault;
	private ModelGraphicWrapperAdv loc;
	/**
	 * Create instance of Application.
	 * @param data setup data
	 */
	public Application(Data data) {
	    this(data, true);
	}
    /**
     * Create instance of Application.
     * @param data setup data
     * @param main is main window
     */
    public Application(Data data, boolean main) {
        this.data = data;
        this.main = main;
        if(main) {
            setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        } else {
            setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        }
        addWindowListener(new SuperSquareHandler(SuperSquare.getInstance().getDimX(), SuperSquare.getInstance().getDimY()));
        addComponentListener(new SizeHandler(data, this));
        addMenus();
        showData();
    }
    private void recursiveSetReadonly(BaseGraphic bg, boolean readonly) {
        bg.setReadonly(readonly);
        for(java.awt.Component c : bg.getComponents()) {
            if(c instanceof BaseGraphic) {
                recursiveSetReadonly((BaseGraphic)c, readonly);
            }
        }
    }
    private void adjustFiller(JPanel c, JPanel w, JPanel e, JPanel s)
    {
        int wdif = (int)(getPreferredSize().getWidth() - (w.getPreferredSize().getWidth() + loc.getPreferredSize().getWidth() + w.getPreferredSize().getWidth()));
        if(wdif > 100) {
        	c.getComponent(2).setPreferredSize(new Dimension((wdif - 100) / 2, 100));
        	c.getComponent(3).setPreferredSize(new Dimension((wdif - 100) / 2, 100));
        } else {
        	c.getComponent(2).setPreferredSize(new Dimension(0, 0));
        	c.getComponent(3).setPreferredSize(new Dimension(0, 0));
        }
        int hdif = (int)(getPreferredSize().getHeight() - (loc.getPreferredSize().getHeight() + s.getPreferredSize().getHeight()));
        if(hdif > 100) {
        	c.getComponent(1).setPreferredSize(new Dimension(100, (hdif - 100) / 2));
        	c.getComponent(4).setPreferredSize(new Dimension(100, (hdif - 100) / 2));
        } else {
        	c.getComponent(1).setPreferredSize(new Dimension(0, 0));
        	c.getComponent(4).setPreferredSize(new Dimension(0, 0));
        }
        c.invalidate();
    }
	private void showData() {
        updateLaf();
        if(main) {
            setTitle(data.getModel().getReal().getName() + " - " + data.getModel().getStore());
        } else {
            setTitle(data.getModel().getReal().getName() + " (readonly view)");
        }
        getContentPane().setLayout(new BorderLayout());
        AllGraphicConfig allcfg = data.getConfig();
        ExtModel m = data.getModel();
		SuperSquare.getInstance().setDimX(m.getDimX());
		SuperSquare.getInstance().setDimY(m.getDimY());
        ModelGraphicWrapperAdv mg = new ModelGraphicWrapperAdv(allcfg, m);
        if(data.isReadonly()) {
            recursiveSetReadonly(mg, true);
        }
        loc = mg;
        final JPanel c = new JPanel();
        c.setLayout(new BorderLayout());
        c.add(mg, BorderLayout.CENTER);
        c.add(new FillerGraphic(), BorderLayout.NORTH);
        c.add(new FillerGraphic(), BorderLayout.WEST);
        c.add(new FillerGraphic(), BorderLayout.EAST);
        c.add(new FillerGraphic(), BorderLayout.SOUTH);
        printDefault = mg;
        JScrollPane cc = new JScrollPane(c, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        Dimension dim = mg.getPreferredSize();
        cc.setPreferredSize(new Dimension(dim.width + 6*allcfg.getModelConfig().getMargin(), dim.height + 3*allcfg.getModelConfig().getMargin()));
        getContentPane().add(cc, BorderLayout.CENTER);
        List<BoxPalette> lbp = data.getBoxPalette();
        final JPanel w = new JPanel();
        w.setLayout(new BoxLayout(w, BoxLayout.Y_AXIS));
        for(BoxPalette bp : lbp) {
            BoxPaletteGraphicWrapper bpg = new BoxPaletteGraphicWrapper(allcfg, bp, data);
            w.add(bpg);
        }
        getContentPane().add(new JScrollPane(w, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.WEST);
        List<ComponentPalette> lcmp = data.getComponentPalette();
        final JPanel e = new JPanel();
        e.setLayout(new BoxLayout(e, BoxLayout.Y_AXIS));
        for(ComponentPalette cp : lcmp) {
            ComponentPaletteGraphicWrapper cpg = new ComponentPaletteGraphicWrapper(allcfg, cp, data);
            e.add(cpg);
        }
        getContentPane().add(new JScrollPane(e, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.EAST);
        List<ConnectionPalette> lcnp = data.getConnectionPalette();
        final JPanel s = new JPanel();
        s.setLayout(new BoxLayout(s, BoxLayout.X_AXIS));
        for(ConnectionPalette cp : lcnp) {
            ConnectionPaletteGraphicWrapper cpg = new ConnectionPaletteGraphicWrapper(allcfg, cp, data);
            s.add(cpg);
        }
        getContentPane().add(new JScrollPane(s, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), BorderLayout.SOUTH);
        if(data.getWidth() == 0 || data.getHeight() == 0) {
            setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
        } else {
            setPreferredSize(new Dimension(data.getWidth(), data.getHeight()));
        }
        c.addComponentListener(new ComponentListener() {
			public void componentHidden(ComponentEvent ce) {
			}
			public void componentMoved(ComponentEvent ce) {
			}
			public void componentResized(ComponentEvent ce) {
				adjustFiller(c, w, e, s);
			}
			public void componentShown(ComponentEvent ce) {
			}
        });
        pack();
	}
    private void addMenus() {
        JMenuBar mb = new JMenuBar();
        mb.add(createFileMenu());
        mb.add(createModelMenu());
        mb.add(createBoxPaletteMenu());
        mb.add(createComponentPaletteMenu());
        mb.add(createConnectionPaletteMenu());
        mb.add(createGraphicsPaletteMenu());
        mb.add(createViewMenu());
        mb.add(createHelpMenu());
        setJMenuBar(mb);
    }
	private JMenu createFileMenu() {
        JMenuItem opnset = new JMenuItem("Open setup");
        opnset.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String fnm = (new XmlPersister()).promptLoad(Application.this, data.getModel().getStore(), "vsma_xml", null);
                if(fnm != null && Util.isValidInputFile(fnm)) {
                    try {
                        data = Control.load(fnm);
                        getContentPane().removeAll();
                        showData();
                    } catch (ModelException ex) {
                        JOptionPane.showMessageDialog(Application.this, "Setup file " + fnm + " is not in correct format");
                    }
                } else {
                    JOptionPane.showMessageDialog(Application.this, "Setup file not opened");
                }
            }
        });
        JMenuItem savset = new JMenuItem("Save setup");
        savset.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String fnm = (new XmlPersister()).promptSave(Application.this, data.getModel().getStore(), "vsma_xml", "setup.vsma_xml");
                if(fnm != null && Util.isValidOutputFile(fnm)) {
                    try {
                        Control.save(data, fnm);
                    } catch (ModelException ex) {
                        JOptionPane.showMessageDialog(Application.this, "Unexpected error saving setup file " + fnm);
                    }
                } else {
                    JOptionPane.showMessageDialog(Application.this, "Setup file not saved");
                }
            }
        });
        JMenu lf = new JMenu("Look & Feel");
        List<AppLookAndFeel> lafs = AppLookAndFeel.lookup();
        for(final AppLookAndFeel laf : lafs) {
            JMenuItem lf1 = new JMenuItem(laf.getThemeDisplayName() != null ? laf.getLafDisplayName() + " " + laf.getThemeDisplayName() : laf.getLafDisplayName());
            lf1.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    data.getLaf().setLafClassName(laf.getLafClassName());
                    data.getLaf().setLafDisplayName(laf.getLafDisplayName());
                    data.getLaf().setThemeClassName(laf.getThemeClassName());
                    data.getLaf().setThemeDisplayName(laf.getThemeDisplayName());
                    updateLaf();
                }
            });
            lf.add(lf1);
        }
        JMenuItem thm = new JMenuItem("Edit customizable theme");
        thm.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ThemeEditPanel tep = new ThemeEditPanel(Application.this, AppMetalThemeData.getInstance());
                tep.setVisible(true);
            }
        });
        JMenuItem ext = new JMenuItem("Exit");
        ext.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        JMenu fm = new JMenu("File");
        fm.add(opnset);
        fm.add(savset);
        fm.add(lf);
        fm.add(thm);
        fm.add(ext);
        return fm;
	}
	private JMenu createModelMenu() {
        JMenuItem nw = new JMenuItem("New");
        nw.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String oldloc = data.getModel().getStore();
                data.setModel(new ExtModel(new Model("New model", new OneNumberVersion(1)), new ArrayList<Location>(), 5, 5));
                data.getModel().setStore(Util.combinePath(oldloc, "newmodel.mod_xml"));
                getContentPane().removeAll();
                showData();
            }
        });
        JMenuItem opn = new JMenuItem("Open");
        opn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String fnm = data.getPersist().promptLoad(Application.this, data.getModel().getStore(), "mod_xml", null);
                if(fnm != null && Util.isValidInputFile(fnm)) {
                    openModel(fnm);
                } else {
                    JOptionPane.showMessageDialog(Application.this, "Model not opened");
                }
            }
        });
        JMenuItem sav = new JMenuItem("Save");
        if(!data.isReadonly()) {
            sav.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    saveModel(data.getModel().getStore());
                }
            });
        } else {
            sav.setEnabled(false);
        }
        JMenuItem savas = new JMenuItem("Save As");
        if(!data.isReadonly()) {
            savas.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String fnm = data.getPersist().promptSave(Application.this, data.getModel().getStore(), "mod_xml", data.getModel().getStore());
                    if(fnm != null && Util.isValidOutputFile(fnm)) {
                        saveModel(fnm);
                    } else {
                        JOptionPane.showMessageDialog(Application.this, "Model not saved");
                    }
                }
            });
        } else {
            savas.setEnabled(false);
        }
        JMenuItem savgrpgif = saveGrp("GIF");
        JMenuItem savgrpjpg = saveGrp("JPEG");
        JMenuItem savgrppng = saveGrp("PNG");
        JMenuItem exp = new JMenuItem("Expand model");
        if(!data.isReadonly()) {
            exp.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    int oldx = SuperSquare.getInstance().getDimX();
                    int oldy = SuperSquare.getInstance().getDimY();
                    int newx = oldx + 2;
                    int newy = oldy + 2;
                    List<Location> loclst = loc.getLocations();
                    for(int i = 0; i < loclst.size(); i++) {
                        Location loc = loclst.get(i);
                        loc.move(1, 1);
                    }
                    data.getModel().setLocations(loclst);
                    data.getModel().setDimX(newx);
                    data.getModel().setDimY(newy);
                    SuperSquare.getInstance().setDimX(newx);
                    SuperSquare.getInstance().setDimY(newy);
                    getContentPane().removeAll();
                    showData();
                }
            });
        } else {
            exp.setEnabled(false);
        }
        JMenuItem crp = new JMenuItem("Crop model");
        if(!data.isReadonly()) {
            crp.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    int oldx = SuperSquare.getInstance().getDimX();
                    int oldy = SuperSquare.getInstance().getDimY();
                    int lowx = oldx - 1;
                    int highx = 0;
                    int lowy = oldy - 1;
                    int highy = 0;
                    List<Location> loclst = loc.getLocations();
                    for(Location loc : loclst) {
                        lowx = Math.min(lowx, loc.getX());
                        highx = Math.max(highx, loc.getX());
                        lowy = Math.min(lowy, loc.getY());
                        highy = Math.max(highy, loc.getY());
                    }
                    int newx = highx - lowx + 1;
                    int newy = highy - lowy + 1;
                    for(int i = 0; i < loclst.size(); i++) {
                        Location loc = loclst.get(i);
                        loc.move(-lowx, -lowy);
                    }
                    data.getModel().setLocations(loclst);
                    data.getModel().setDimX(newx);
                    data.getModel().setDimY(newy);
                    SuperSquare.getInstance().setDimX(newx);
                    SuperSquare.getInstance().setDimY(newy);
                    getContentPane().removeAll();
                    showData();
                }
            });
        } else {
            crp.setEnabled(false);
        }
        JMenu mm = new JMenu("Model");
        mm.add(nw);
        mm.add(opn);
        mm.add(sav);
        mm.add(savas);
        mm.add(savgrpgif);
        mm.add(savgrpjpg);
        mm.add(savgrppng);
        mm.add(exp);
        mm.add(crp);
        return mm;
	}
	private JMenu createBoxPaletteMenu() {
        JMenuItem savbox = new JMenuItem("Save model as box palette");
        savbox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Model m = data.getModel().getReal();
                BoxPalette pal = new BoxPalette(m.getName(), new TwoNumberVersion(1, 0));
                for(Box b : m.getBoxes()) {
                    if(!pal.getBoxes().contains(b)) {
                        pal.addBox(b);
                    }
                }
                String fnm = data.getPersist().promptSave(Application.this, data.getModel().getStore(), "boxpal_xml", data.getModel().getReal().getName() + ".boxpal_xml");
                if(fnm != null && Util.isValidOutputFile(fnm)) {
                    try {
                        data.getPersist().save(pal, fnm);
                    } catch (ModelException ex) {
                        JOptionPane.showMessageDialog(Application.this, "Unexpected error saving palette to " + fnm);
                    }
                } else {
                    JOptionPane.showMessageDialog(Application.this, "Model not saved as box palette");
                }
            }
        });
        JMenuItem loadbox = loadPalette("box", data.getBoxPalette(), "box");
        JMenu rembox = removePalette("box", data.getBoxPalette());
        JMenuItem edtbp = editPalette("box", data.getBoxPalette());
        JMenuItem savbp = savePalette("box", data.getBoxPalette());
        JMenu bpm = new JMenu("Boxes");
        bpm.add(savbox);
        bpm.add(loadbox);
        bpm.add(rembox);
        bpm.add(edtbp);
        bpm.add(savbp);
        return bpm;
	}
	private JMenu createComponentPaletteMenu() {
       JMenuItem savcomp = new JMenuItem("Save model as component palette");
        savcomp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Model m = data.getModel().getReal();
                ComponentPalette pal = new ComponentPalette(m.getName(), new TwoNumberVersion(1, 0));
                for(Box b : m.getBoxes()) {
                    for(Component c : b.getComponents()) {
                        if(!pal.getComponents().contains(c)) {
                            pal.addComponent(c);
                        }
                    }
                }
                String fnm = data.getPersist().promptSave(Application.this, data.getModel().getStore(), "comppal_xml", data.getModel().getReal().getName() + ".comppal_xml");
                if(fnm != null && Util.isValidOutputFile(fnm)) {
                    try {
                        data.getPersist().save(pal, fnm);
                    } catch (ModelException ex) {
                        JOptionPane.showMessageDialog(Application.this, "Unexpected error saving palette to " + fnm);
                    }
                } else {
                    JOptionPane.showMessageDialog(Application.this, "Model not saved as component palette");
                }
            }
        });
        JMenuItem loadcomp = loadPalette("component", data.getComponentPalette(), "comp");
        JMenu remcomp = removePalette("component", data.getComponentPalette());
        JMenuItem edtcmp = editPalette("component", data.getComponentPalette());
        JMenuItem savcmp = savePalette("component", data.getComponentPalette());
        JMenu cmpm = new JMenu("Components");
        cmpm.add(savcomp);
        cmpm.add(loadcomp);
        cmpm.add(remcomp);
        cmpm.add(edtcmp);
        cmpm.add(savcmp);
        return cmpm;
	}
	private JMenu createConnectionPaletteMenu() {
        JMenuItem savconn = new JMenuItem("Save model as connection palette");
        savconn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Model m = data.getModel().getReal();
                ConnectionPalette pal = new ConnectionPalette(m.getName(), new TwoNumberVersion(1, 0));
                for(Connection c : m.getConnections()) {
                    if(!pal.getConnections().contains(c)) {
                        pal.addConnection(c);
                    }
                }
                String fnm = data.getPersist().promptSave(Application.this, data.getModel().getStore(), "connpal_xml", data.getModel().getReal().getName() + ".connpal_xml");
                if(fnm != null && Util.isValidOutputFile(fnm)) {
                    try {
                        data.getPersist().save(pal, fnm);
                    } catch (ModelException ex) {
                        JOptionPane.showMessageDialog(Application.this, "Unexpected error saving palette to " + fnm);
                    }
                } else {
                    JOptionPane.showMessageDialog(Application.this, "Model not saved as connection palette");
                }
            }
        });
        JMenuItem loadconn = loadPalette("connection", data.getConnectionPalette(), "conn");
        JMenu remconn = removePalette("connection", data.getConnectionPalette());
        JMenuItem edtcnp = editPalette("connection", data.getConnectionPalette());
        JMenuItem savcnp = savePalette("connection", data.getConnectionPalette());
        JMenu cnpm = new JMenu("Connections");
        cnpm.add(savconn);
        cnpm.add(loadconn);
        cnpm.add(remconn);
        cnpm.add(edtcnp);
        cnpm.add(savcnp);
        return cnpm;
	}
	private JMenu createGraphicsPaletteMenu() {
        JMenu grpm = new JMenu("Graphics");
        JMenuItem mgrp = grpEdit("Model config", data.getConfig().getModelConfig());
        grpm.add(mgrp);
        JMenuItem pgrp = grpEdit("Palette config", data.getConfig().getPaletteConfig());
        grpm.add(pgrp);
        for(String k : data.getConfig().getBoxes()) {
            JMenuItem bgrp = grpEdit(k + " box config", data.getConfig().getBoxConfig(k));
            grpm.add(bgrp);
        }
        JMenuItem newbgrp = new JMenuItem("New box config");
        newbgrp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String vendor = JOptionPane.showInputDialog(Application.this, "Enter vendor name");
                GraphicConfig cfg = (GraphicConfig)data.getConfig().getStandardConfig().clone();
                GraphicEditPanel p = new GraphicEditPanel(Application.this, cfg);
                p.setVisible(true);
                data.getConfig().setBoxConfig(vendor, cfg);
                getContentPane().removeAll();
                showData();
            }
        });
        grpm.add(newbgrp);
        for(String k : data.getConfig().getComponents()) {
            JMenuItem cmgrp = grpEdit(k + " component config", data.getConfig().getComponentConfig(k));
            grpm.add(cmgrp);
        }
        JMenuItem newcgrp = new JMenuItem("New component config");
        newcgrp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String vendor = JOptionPane.showInputDialog(Application.this, "Enter vendor name");
                GraphicConfig cfg = (GraphicConfig)data.getConfig().getStandardConfig().clone();
                GraphicEditPanel p = new GraphicEditPanel(Application.this, cfg);
                p.setVisible(true);
                data.getConfig().setComponentConfig(vendor, cfg);
                getContentPane().removeAll();
                showData();
            }
        });
        grpm.add(newcgrp);
        JMenuItem cngrp = grpEdit("Connections config", data.getConfig().getConnectionConfig());
        grpm.add(cngrp);
        JMenuItem dfgrp = grpEdit("Default config", data.getConfig().getStandardConfig());
        grpm.add(dfgrp);
        JMenuItem savgrp = new JMenuItem("Save");
        savgrp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    data.getPersist().save(data.getConfig());
                } catch (ModelException ex) {
                    JOptionPane.showMessageDialog(Application.this, "Unexpected error saving graphic");
                }
            }
        });
        grpm.add(savgrp);
        return grpm;
	}
    private JMenu createViewMenu() {
        final DisplayFilter[] allfilter = new DisplayFilter[] { new IgnoreVMsFilter(),
                                                                new HideHardwareDetailsFilter(),
                                                                new FlattenSoftwareComponentsFilter(),
                                                                new HideConnectionsFilter(),
                                                                new RemoveDuplicatesFilter(),
                                                                new GrayPassiveComponentsFilter(),
                                                                new PackFilter() };
        JMenu vm = new JMenu("View");
        for(final DisplayFilter filter : allfilter) {
            JMenuItem v = new JMenuItem(filter.getName());
            v.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Data data2 = filter.filter(data);
                    data2.setReadonly(true);
                    Application app2 = new Application(data2, false);
                    app2.setVisible(true);
                }
            });
            vm.add(v);
        }
        JMenuItem multi = new JMenuItem("Multi");
        multi.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                MultiFilterPanel p = new MultiFilterPanel(Application.this, allfilter, data);
                p.setVisible(true);
            }
        });
        vm.add(multi);
        return vm;
    }
    private JMenu createHelpMenu() {
        JMenuItem hlp = new JMenuItem("Help");
        hlp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JDialog hlpwin = new HelpPanel(Application.this);
                hlpwin.setVisible(true);
            }
        });
        JMenuItem scr = new JMenuItem("Scripting");
        scr.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JDialog scrwin = new ScriptPanel(Application.this, data);
                scrwin.setVisible(true);
                getContentPane().removeAll();
                showData();
            }
        });
        JMenuItem abt = new JMenuItem("About");
        abt.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(Application.this, "VSMA (Very Simple Model Application)\r\nVersion: " + VERSION + "\r\nAuthor: Arne Vajhj");
            }
        });
        JMenu hm = new JMenu("Help");
        hm.add(hlp);
        hm.add(scr);
        hm.add(abt);
        return hm;
    }
    private void openModel(String fnm) {
        try {
            data.setModel((ExtModel)data.getPersist().load(fnm, ExtModel.class));
            getContentPane().removeAll();
            showData();
		} catch (ModelException ex) {
		    JOptionPane.showMessageDialog(Application.this, "Model in " + fnm + " is not in correct format");
		}
    }
    private void saveModel(String fnm) {
        try {
            data.getPersist().save(new ExtModel(data.getModel().getReal(), loc.getLocations(), data.getModel().getDimX(), data.getModel().getDimY()), fnm);
            data.getModel().setStore(fnm);
		} catch (ModelException ex) {
            JOptionPane.showMessageDialog(Application.this, "Unexpected error saving model to " + fnm);
		}
    }
    private void printModel(String fnm, String fmt) {
        try {
            BufferedImage bi = new BufferedImage(printDefault.getWidth(), printDefault.getHeight(), BufferedImage.TYPE_INT_RGB);
            Graphics g = bi.createGraphics();
            printDefault.printAll(g);
			ImageIO.write(bi, fmt, new File(fnm));
		} catch (IOException ex) {
            JOptionPane.showMessageDialog(Application.this, "Unexpected error saving image file " + fnm);
		}
    }
	private JMenuItem saveGrp(final String fmt) {
		JMenuItem mi = new JMenuItem("Save " + fmt);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    String fnm = data.getPersist().promptSave(Application.this, data.getModel().getStore(), fmt.toLowerCase(), data.getModel().getReal().getName() + "." + fmt.toLowerCase());
			    if(fnm != null && Util.isValidOutputFile(fnm)) {
                    printModel(fnm, fmt);
			    } else {
                    JOptionPane.showMessageDialog(Application.this, "Graphic image not saved");
                }
			}
		});
		return mi;
	}
	private <T extends PaletteBase> JMenuItem loadPalette(final String typ, final List<T> lst, final String shorttyp) {
		JMenuItem mi = new JMenuItem("Load new " + typ + " palette");
		mi.addActionListener(new ActionListener() {
			@SuppressWarnings("unchecked")
			public void actionPerformed(ActionEvent e) {
			    String fnm = data.getPersist().promptLoad(Application.this, data.getModel().getStore(), shorttyp + "pal_xml", null);
			    if(fnm != null && Util.isValidInputFile(fnm)) {
                    try {
                        lst.add((T)data.getPersist().load(fnm, SeparatePersistable.class));
                        data.getShowPalette().put(fnm, true);
                        getContentPane().removeAll();
                        showData();
                    } catch (ModelException ex) {
                        JOptionPane.showMessageDialog(Application.this, "Palette in " + fnm + " is not in correct format");
                    }
		        } else {
                    JOptionPane.showMessageDialog(Application.this, "Palette not loaded");
                }
			}
		});
		return mi;
	}
	private <T extends PaletteBase> JMenu removePalette(String typ, final List<T> lst) {
		JMenu m = new JMenu("Remove " + typ + " palette");
		for(final T p : lst) {
			JMenuItem remonebox = new JMenuItem(p.getName());
			remonebox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					lst.remove(p);
		            getContentPane().removeAll();
		            showData();
				}
			});
			m.add(remonebox);
		}
		return m;
	}
    private <T extends PaletteBase> JMenuItem savePalette(final String typ, final List<T> lst) {
        JMenuItem mi = new JMenuItem("Save all " + typ + " palette");
        mi.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                for(final T p : lst) {
                    try {
                        data.getPersist().save((SeparatePersistable) p);
                    } catch (ModelException ex) {
                        JOptionPane.showMessageDialog(Application.this, "Unexpected error saving palette");
                    }
                }
            }
        });
        return mi;
    }
    private <T extends PaletteBase> JMenu editPalette(final String typ, final List<T> lst) {
        JMenu m = new JMenu("Edit " + typ + " palette");
        for(final T p : lst) {
            JMenuItem edtonebox = new JMenuItem(p.getName());
            edtonebox.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String oldloc = data.getModel().getStore();
                    int n = 5;
                    int x = (int)Math.ceil(Math.sqrt(n));
                    int y = (n - 1) / x + 1;
                    int ix = 0;
                    data.setModel(new ExtModel(new Model(p.getName(), p.getVersion().bumpLeast()), new ArrayList<Location>(), x, y));
                    data.getModel().setStore(Util.combinePath(oldloc, p.getName() + ".mod_xml"));
                    if(p instanceof BoxPalette) {
                        BoxPalette bp = ((BoxPalette)p).duplicate();
                        for(Box b : bp.getBoxes()) {
                            b.setId(ix);
                            data.getModel().getReal().addBox(b);
                            data.getModel().getLocations().add(new Location(ix % x, ix / x, BoxGraphicWrapper.class.getName(), b.getId()));
                            ix++;
                        }
                    } else if(p instanceof ComponentPalette) {
                        ComponentPalette cp = ((ComponentPalette)p).duplicate();
                        Box b = new Box("Container", new StringVersion(""), "", "");
                        b.setId("Container");
                        for(Component c : cp.getComponents()) {
                            b.addComponent(c);
                        }
                        data.getModel().getReal().addBox(b);
                        data.getModel().getLocations().add(new Location(0, 0, BoxGraphicWrapper.class.getName(), "Container"));
                    } else if(p instanceof ConnectionPalette) {
                        ConnectionPalette cp = ((ConnectionPalette)p).duplicate();
                        for(Connection c : cp.getConnections()) {
                            c.setId(ix);
                            data.getModel().getReal().addConnection(c);
                            data.getModel().getLocations().add(new Location(ix % x, ix / x, ConnectionGraphicWrapper.class.getName(), c.getId()));
                            ix++;
                        }
                    } else {
                        throw new ModelRuntimeException("Unknown palette type: " + p.getClass().getName());
                    }
                    getContentPane().removeAll();
                    showData();
                }
            });
            m.add(edtonebox);
        }
        return m;
    }
	private JMenuItem grpEdit(String lbl, final GraphicConfig cfg) {
		JMenuItem mi = new JMenuItem(lbl);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				GraphicEditPanel p = new GraphicEditPanel(Application.this, cfg);
				p.setVisible(true);
	            getContentPane().removeAll();
	            showData();
			}
		});
		return mi;
	}
    private void updateLaf() {
        try {
            if(data.getLaf().getLafClassName().equals("javax.swing.plaf.metal.MetalLookAndFeel") && data.getLaf().getThemeClassName() != null) {
                MetalLookAndFeel.setCurrentTheme((MetalTheme)Class.forName(data.getLaf().getThemeClassName()).newInstance());
            }
            UIManager.setLookAndFeel(data.getLaf().getLafClassName());
            SwingUtilities.updateComponentTreeUI(Application.this);
        } catch (InstantiationException e) {
            throw new ModelRuntimeException("Error loading L&F", e);
        } catch (IllegalAccessException e) {
            throw new ModelRuntimeException("Error loading L&F", e);
        } catch (ClassNotFoundException e) {
            throw new ModelRuntimeException("Error loading L&F", e);
        } catch (UnsupportedLookAndFeelException e) {
            throw new ModelRuntimeException("Error loading L&F", e);
        }
    }
}

class SuperSquareHandler implements WindowListener {
    private int dimx;
    private int dimy;
    public SuperSquareHandler(int dimx, int dimy) {
        this.dimx = dimx;
        this.dimy = dimy;
    }
    public void windowOpened(WindowEvent e) {
    }
    public void windowClosing(WindowEvent e) {
    }
    public void windowClosed(WindowEvent e) {
        SuperSquare.getInstance().setDimX(dimx);
        SuperSquare.getInstance().setDimY(dimy);
    }
    public void windowIconified(WindowEvent e) {
    }
    public void windowDeiconified(WindowEvent e) {
    }
    public void windowActivated(WindowEvent e) {
    }
    public void windowDeactivated(WindowEvent e) {
    }
}

class SizeHandler implements ComponentListener {
    private Data data;
    private Application app;
    public SizeHandler(Data data, Application app) {
        this.data = data;
        this.app = app;
    }
    public void componentResized(ComponentEvent e) {
        data.setWidth(app.getWidth());
        data.setHeight(app.getHeight());
    }
    public void componentMoved(ComponentEvent e) {
    }
    public void componentShown(ComponentEvent e) {
    }
    public void componentHidden(ComponentEvent e) {
    }
}