/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.util.List;
import java.util.Map;

import dk.vajhoej.model.Persister;
import dk.vajhoej.model.core.BoxPalette;
import dk.vajhoej.model.core.ComponentPalette;
import dk.vajhoej.model.core.ConnectionPalette;
import dk.vajhoej.model.core.ExtModel;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class Data contains the data from an setup XML file.
 */
public class Data {
	private Persister persist;
	private AppLookAndFeel laf;
	private int width;
	private int height;
	private AllGraphicConfig config;
	private List<BoxPalette> boxPalette;
	private List<ComponentPalette> componentPalette;
	private List<ConnectionPalette> connectionPalette;
	private Map<String, Boolean> showPalette;
	private ExtModel model;
	private boolean readonly;
	/**
	 * Create instance of Data.
	 * @param persist persister
	 * @param laf L&F
	 * @param config all graphic configurations
	 * @param boxPalette all box palettes
	 * @param componentPalette all component palettes
	 * @param connectionPalette all connection palettes
	 * @param showPalette show palettes
	 * @param model the model
	 */
	public Data(Persister persist, AppLookAndFeel laf, int width, int height, AllGraphicConfig config, List<BoxPalette> boxPalette, List<ComponentPalette> componentPalette, List<ConnectionPalette> connectionPalette, Map<String, Boolean> showPalette, ExtModel model) {
		super();
		this.persist = persist;
		this.laf = laf;
		this.width = width;
		this.height = height;
		this.config = config;
		this.boxPalette = boxPalette;
		this.componentPalette = componentPalette;
		this.connectionPalette = connectionPalette;
		this.showPalette = showPalette;
		this.model = model;
		this.readonly = false;
	}
	/**
	 * Get persister.
	 * @return persister.
	 */
	public Persister getPersist() {
		return persist;
	}
	/**
	 * Get L&F.
	 * @return L&F
	 */
	public AppLookAndFeel getLaf() {
	    return laf;
	}
    /**
     * Get width.
     * @return width
     */
    public int getWidth() {
        return width;
    }
    /**
     * Set width.
     * @param width width
     */
    public void setWidth(int width) {
        this.width = width;
    }
    /**
     * Get height.
     * @return height
     */
    public int getHeight() {
        return height;
    }
    /**
     * Set height.
     * @param height height
     */
    public void setHeight(int height) {
        this.height = height;
    }
	/**
	 * Get all graphic configuration.
	 * @return all graphic configuration
	 */
	public AllGraphicConfig getConfig() {
		return config;
	}
	/**
	 * Get all box palettes.
	 * @return all box palettes
	 */
	public List<BoxPalette> getBoxPalette() {
		return boxPalette;
	}
	/**
	 * Get all component palettes.
	 * @return all component palettes 
	 */
	public List<ComponentPalette> getComponentPalette() {
		return componentPalette;
	}
	/**
	 * Get all connection palettes.
	 * @return all connection palettes
	 */
	public List<ConnectionPalette> getConnectionPalette() {
		return connectionPalette;
	}
	/**
	 * Get show palettes.
	 * @return show palettes
	 */
	public Map<String, Boolean> getShowPalette() {
	    return showPalette;
	}
	/**
	 * Get the model.
	 * @return the model
	 */
	public ExtModel getModel() {
		return model;
	}
	/**
	 * Set the model.
	 * @param model the model
	 */
	public void setModel(ExtModel model) {
		this.model = model;
	}
	/**
	 * Get readonly status.
	 * @return readonly status
	 */
	public boolean isReadonly() {
	    return readonly;
	}
	/**
	 * Set readonly status.
	 * @param readonly readonly status
	 */
	public void setReadonly(boolean readonly) {
	    this.readonly = readonly;
	}
	// basic
	@Override
	public String toString() {
	    return "(" + persist + "," + laf + "," + config + "," + boxPalette + "," + componentPalette + "," + connectionPalette + "," + showPalette + "," + model + ")"; 
	}
}
