/*
 * Copyright 2011 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;

/**
 * Class FilenameShowWrapper wraps a filename and a show status.
 */
public class FilenameShowWrapper implements Persistable {
    private String filename;
    private boolean show;
    /**
     * Create instance of FilenameShowWrapper.
     */
    public FilenameShowWrapper() {
        this("", false);
    }
    /**
     * Create instance of FilenameShowWrapper.
     * @param filename filename
     * @param show show status
     */
    public FilenameShowWrapper(String filename, boolean show) {
        this.filename = filename;
        this.show = show;
    }
    /**
     * Get filename.
     * @return filename
     */
    public String getFilename() {
        return filename;
    }
    /**
     * Get show status.
     * @return show status
     */
    public boolean isShow() {
        return show;
    }
    // Persistable
    public String getId() {
        return filename;
    }
    public String getType() {
        return "FilenameShowWrapper";
    }
    public NameValuePairList getAttributes() {
        NameValuePairList res = new NameValuePairList();
        res.put("filename", filename);
        res.put("show", Boolean.toString(show));
        return res;
    }
    public List<List<Persistable>> getChildren() {
        return new ArrayList<List<Persistable>>();
    }
    public void setAttributes(NameValuePairList attr) throws ModelException {
        filename = attr.get("filename");
        show = Boolean.parseBoolean(attr.get("show"));
    }
    public void setChildren(List<List<Persistable>> childs) throws ModelException {
    }
}