/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.Util;
import dk.vajhoej.model.persist.XmlPersister;

/**
 * Class Main contains the main method.
 */
public class Main {
    private static final String DEFAULT_SETUP_FILE = "setup.vsma_xml";
    /**
     * Main method.
     * @param args command line arguments (first argument = path to setup XML file,
     * if not specified then user is prompted.
     */
    public static void main(String[] args) {
        final String fnm;
        if(args.length > 0 && Util.isValidInputFile(args[0])) {
            fnm = args[0];
        } else if(Util.isValidInputFile(DEFAULT_SETUP_FILE)) {
            fnm = DEFAULT_SETUP_FILE;
        } else {
            fnm = (new XmlPersister()).promptLoad(null, System.getProperty("user.dir"), "vsma_xml", null);
        }
    	if(fnm != null) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    try {
                        Data data = Control.load(fnm);
                        JFrame f = new Application(data);
                        f.setVisible(true);
                    } catch (ModelException e) {
                        JOptionPane.showMessageDialog(null, "Setup file " + fnm + " is not in correct format");
                    }
                }
            });
    	} else {
            JOptionPane.showMessageDialog(null, "No setup file chosen - exiting.");
    	}
    }
}
