/*
 * Copyright 2011 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.core.ExtModel;
import dk.vajhoej.model.core.Location;

/**
 * Class PackFilter packs everything.
 */
public class PackFilter implements DisplayFilter {
    public Data filter(Data d) {
        int n = d.getModel().getReal().getBoxes().size() + d.getModel().getReal().getConnections().size();
        int x = (int)Math.ceil(Math.sqrt(n));
        int y = (n - 1) / x + 1;
        List<Location> allloc = new ArrayList<Location>();
        int ix = 0;
        for(Location loc : d.getModel().getLocations()) {
            allloc.add(new Location(ix % x, ix / x, loc.getClazz(), loc.getId()));
            ix++;
        }
        Data res = new Data(d.getPersist(),
                            d.getLaf(),
                            d.getWidth(),
                            d.getHeight(),
                            d.getConfig().clone(),
                            d.getBoxPalette(),
                            d.getComponentPalette(),
                            d.getConnectionPalette(),
                            d.getShowPalette(),
                            new ExtModel(d.getModel().getReal().duplicate(),
                                         allloc,
                                         x,
                                         y));
        return res;
    }
    public String getName() {
        return "Pack";
    }
}
