/*
 * Copyright 2011 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;

/**
 * Class Setup2 is the in memory representation of a setup XML file in the new format.
 */
public class Setup2 extends Setup {
	private static final long serialVersionUID = 1L;
	// part Persistable
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("presentation", presentation);
		res.put("persister", persister);
		res.put("persisterParams", (persisterParams != null) ? persisterParams : "");
		if(laf.size() == 2) {
            res.put("laf", laf.get(0) + " " + laf.get(1));
		} else if(laf.size() == 4) {
            res.put("laf", laf.get(0) + " " + laf.get(1) + " " + laf.get(2) + " " + laf.get(3));
		} else {
		    throw new ModelRuntimeException("Invalid L&F in setup file");
		}
        res.put("w", Integer.toString(w));
        res.put("h", Integer.toString(h));
		res.put("model", model);
        AppMetalThemeData theme = AppMetalThemeData.getInstance();
        res.put("metalTheme", Integer.toHexString(theme.getBorder().getRGB()).substring(2) + " " +
                              Integer.toHexString(theme.getMarked().getRGB()).substring(2) + " " +
                              Integer.toHexString(theme.getBackground().getRGB()).substring(2));
		return res;
	}
	public List<List<Persistable>> getChildren() {
        List<Persistable> bp = new ArrayList<Persistable>();
        for(int i = 0; i < boxes.size(); i++) {
            bp.add(new FilenameShowWrapper(boxes.get(i), showBoxes.get(i)));
        }
        List<Persistable> cmp = new ArrayList<Persistable>();
        for(int i = 0; i < components.size(); i++) {
            cmp.add(new FilenameShowWrapper(components.get(i), showComponents.get(i)));
        }
        List<Persistable> cnp = new ArrayList<Persistable>();
        for(int i = 0; i < connections.size(); i++) {
            cnp.add(new FilenameShowWrapper(connections.get(i), showConnections.get(i)));
        }
	    List<List<Persistable>> res = new ArrayList<List<Persistable>>();
        res.add(bp);
        res.add(cmp);
        res.add(cnp);
	    return res;
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		presentation = attr.get("presentation");
		persister = attr.get("persister");
		persisterParams = (attr.get("persisterParams") != null) ? attr.get("persister") : "";
		if(attr.get("laf") != null) {
		    for(String s : attr.get("laf").split(" ")) {
		        laf.add(s);
		    }
		} else {
            laf.add("javax.swing.plaf.metal.MetalLookAndFeel");
            laf.add("Metal");
		}
        if(attr.get("w") != null) {
            w = Integer.parseInt(attr.get("w"));
        } else {
            w = 0;
        }
        if(attr.get("h") != null) {
            h = Integer.parseInt(attr.get("h"));
        } else {
            h = 0;
        }
		model = attr.get("model");
        if(attr.get("metalTheme") != null) {
            String[] cols = attr.get("metalTheme").split(" ");
            AppMetalThemeData theme = AppMetalThemeData.getInstance();
            theme.setBorder(Color.decode("0x" + cols[0]));
            theme.setMarked(Color.decode("0x" + cols[1]));
            theme.setBackground(Color.decode("0x" + cols[2]));
        }
	}
	public void setChildren(List<List<Persistable>> childs) throws ModelException {
        for(Persistable p : childs.get(0)) {
            FilenameShowWrapper fsw = (FilenameShowWrapper)p;
            boxes.add(fsw.getFilename());
            showBoxes.add(fsw.isShow());
        }
        for(Persistable p : childs.get(1)) {
            FilenameShowWrapper fsw = (FilenameShowWrapper)p;
            components.add(fsw.getFilename());
            showComponents.add(fsw.isShow());
        }
        for(Persistable p : childs.get(2)) {
            FilenameShowWrapper fsw = (FilenameShowWrapper)p;
            connections.add(fsw.getFilename());
            showConnections.add(fsw.isShow());
        }
	}
}

