/*
 * Copyright 2011 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

/**
 * Class ThemeEditPanel contains the necesarry functionality to
 * edit an AppMetalThemeData object.
 */
public class ThemeEditPanel extends JDialog {
	private static final long serialVersionUID = 1L;
	private Color oldborder;
	private Color oldmarked;
	private Color oldbackground;
	/**
	 * Create instance of ThemeEditPanel.
	 * @param parent parent frame
	 * @param theme theme to edit
	 */
	public ThemeEditPanel(JFrame parent, final AppMetalThemeData theme) {
		super(parent, "Edit theme", true);
		setLayout(new BorderLayout());
        oldborder = theme.getBorder();
        oldmarked = theme.getMarked();
        oldbackground = theme.getBackground();
		JPanel cols = new JPanel();
		cols.setLayout(new GridLayout(3, 1, 10, 10));
        cols.setBorder(new EmptyBorder(5, 5, 5, 5));
        JButton bordbtn = new JButton("Border color");
        bordbtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color c = JColorChooser.showDialog(ThemeEditPanel.this, "Choose border color", theme.getBorder());
                if(c != null) {
                    theme.setBorder(c);
                }
            }
        });
        cols.add(bordbtn);
        JButton markbtn = new JButton("Marker color");
        markbtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color c = JColorChooser.showDialog(ThemeEditPanel.this, "Choose marker color", theme.getMarked());
                if(c != null) {
                    theme.setMarked(c);
                }
            }
        });
        cols.add(markbtn);
        JButton backbtn = new JButton("Background color");
        backbtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Color c = JColorChooser.showDialog(ThemeEditPanel.this, "Choose background color", theme.getBackground());
                if(c != null) {
                    theme.setBackground(c);
                }
            }
        });
        cols.add(backbtn);
		add(cols, BorderLayout.CENTER);
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ThemeEditPanel.this.dispose();
			}
		});
		JButton can = new JButton("Cancel");
		can.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
                theme.setBorder(oldborder);
                theme.setMarked(oldmarked);
                theme.setBackground(oldbackground);
				ThemeEditPanel.this.dispose();
			}
		});
		JPanel btns = new JPanel();
		btns.setLayout(new GridLayout(1, 2, 10, 10));
		btns.setBorder(new EmptyBorder(5, 5, 5, 5));
		btns.add(ok);
		btns.add(can);
		add(btns, BorderLayout.SOUTH);
		pack();
	}
}
