/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

/**
 * Enum BoxType specifies type of hardware box.
 * <br>
 * Semantics:
 * <table border="yes">
 * <tr>
 * <th>enum value</th>
 * <th>description</th>
 * </tr>
 * <tr>
 * <td>SINGLE_SYS</td>
 * <td>Single system</td>
 * </tr>
 * <tr>
 * <td>MULTI_SYS</td>
 * <td>Multiple systems</td>
 * </tr>
 * <tr>
 * <td>SINGLE_DISK</td>
 * <td>Single disk system</td>
 * </tr>
 * <tr>
 * <td>MULTI_DISK</td>
 * <td>Multiple disk systems</td>
 * </tr>
 * <tr>
 * <td>SINGLE_FIREWALL</td>
 * <td>Single firewall</td>
 * </tr>
 * <tr>
 * <td>MULTI_FIREWALL</td>
 * <td>Multiple firewalls</td>
 * </tr>
 * <tr>
 * <td>SINGLE_LOADBALANCER</td>
 * <td>Single load balancer</td>
 * </tr>
 * <tr>
 * <td>MULTI_LOADBALANCER</td>
 * <td>Multiple load balancers</td>
 * </tr>
 * </table>
 */
public enum BoxType {
	SINGLE_SYS,
	MULTI_SYS,
	SINGLE_DISK,
	MULTI_DISK,
    SINGLE_FIREWALL,
    MULTI_FIREWALL,
    SINGLE_LOADBALANCER,
    MULTI_LOADBALANCER;
	@Override
	public String toString() {
		switch(this) {
			case SINGLE_SYS:
				return "Single system";
			case MULTI_SYS:
				return "Multiple systems";
			case SINGLE_DISK:
				return "Single disk system";
			case MULTI_DISK:
				return "Multipel disk systems";
            case SINGLE_FIREWALL:
                return "Single firewall";
            case MULTI_FIREWALL:
                return "Multiple firewalls";
            case SINGLE_LOADBALANCER:
                return "Single load balancer";
            case MULTI_LOADBALANCER:
                return "Multiple load balancers";
			default:
				return "Unknown";
		}
	}
	/**
	 * Parse string with box type.
	 * @param s string with box type
	 * @return box type
	 */
	public static BoxType parse(String s) {
		for(BoxType b : BoxType.values()) {
			if(b.toString().equals(s)) {
				return b;
			}
		}
		return null;
	}
}
