/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

/**
 * Enum ComponentType specifies type of software component.
 * <br>
 * Semantics:
 * <table border="yes">
 * <tr>
 * <th>enum value</th>
 * <th>description</th>
 * </tr>
 * <tr>
 * <td>APP</td>
 * <td>Application</td>
 * </tr>
 * <tr>
 * <td>APPUML</td>
 * <td>Application (UML style)</td>
 * </tr>
 * <tr>
 * <td>DB</td>
 * <td>Database instance</td>
 * </tr>
 * <tr>
 * <td>VM</td>
 * <td>Virtual Machine</td>
 * </tr>
 * </table>
 */
public enum ComponentType {
	APP,
	APPUML,
	DB,
	VM;
	@Override
	public String toString() {
		switch(this) {
            case APP:
                return "Application";
            case APPUML:
                return "Application (UML style)";
			case DB:
				return "Database";
			case VM:
				return "Virtual Machine";
			default:
				return "Unknown";
		}
	}
	/**
	 * Parse string with component type.
	 * @param s string with component type
	 * @return component type
	 */
	public static ComponentType parse(String s) {
		for(ComponentType c : ComponentType.values()) {
			if(c.toString().equals(s)) {
				return c;
			}
		}
		return null;
	}
}
