/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

/**
 * Enum ConnectionType specifies type of connection between boxes.
 * <br>
 * Semantics:
 * <table border="yes">
 * <tr>
 * <th>enum value</th>
 * <th>description</th>
 * </tr>
 * <tr>
 * <td>SINGLE</td>
 * <td>One protocol</td>
 * </tr>
 * <tr>
 * <td>DOUBLE</td>
 * <td>Two protocols</td>
 * </tr>
 * <tr>
 * <td>TRIPPLE</td>
 * <td>Three protocols</td>
 * </tr>
 * <tr>
 * <td>LEFTBUS</td>
 * <td>Left end of bus</td>
 * </tr>
 * <tr>
 * <td>MIDBUS</td>
 * <td>Middle of bus</td>
 * </tr>
 * <tr>
 * <td>RIGHTBUS</td>
 * <td>Right end of bus</td>
 * </tr>
 * <tr>
 * <td>MIDCROSS</td>
 * <td>Middle cross</td>
 * </tr>
 * <tr>
 * <td>ENDCROSS</td>
 * <td>End cross</td>
 * </tr>
 * </table>
 */
public enum ConnectionType {
	SINGLE,
	DOUBLE,
	TRIPPLE,
	LEFTBUS,
	MIDBUS,
	RIGHTBUS,
	MIDCROSS,
	ENDCROSS;
	@Override
	public String toString() {
		switch(this) {
			case SINGLE:
				return "One protocol";
			case DOUBLE:
				return "Two protocols";
            case TRIPPLE:
                return "Three protocols";
            case LEFTBUS:
                return "Left of bus";
            case MIDBUS:
                return "Middle of bus";
            case RIGHTBUS:
                return "Right of bus";
            case MIDCROSS:
                return "Middle cross";
            case ENDCROSS:
                return "End cross";
			default:
				return "Unknown";
		}
	}
	/**
	 * Parse string with component type.
	 * @param s string with component type
	 * @return component type
	 */
	public static ConnectionType parse(String s) {
		for(ConnectionType c : ConnectionType.values()) {
			if(c.toString().equals(s)) {
				return c;
			}
		}
		return null;
	}
}
