/*
 * Copyright 2012 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.Duplicatable;
import dk.vajhoej.model.FileBrowse;
import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;

/**
 * Class Picture represents a picture with no functionality.
 */
public class Picture implements Persistable, Duplicatable, Serializable {
	private static final long serialVersionUID = 1L;
	@FileBrowse(directory="pic")
	private String filename;
	private String id;
	/**
     * Create instance of Picture.
	 */
	public Picture() {
		this("");
	}
	/**
     * Create instance of Picture.
	 * @param picfnm picture filename
	 */
	public Picture(String picfnm) {
	    this.filename = picfnm;
        id = "tmppicture#" + System.currentTimeMillis();
	}
	/**
	 * Get picture filename.
	 * @return picture filename.
	 */
	public String getFilename() {
		return filename;
	}
	/**
	 * Set picture filename.
	 * @param filename picture filename
	 */
	public void setFilename(String filename) {
	    this.filename = filename;
	}
    /**
     * Set ID.
     * @param id id
     */
    public void setId(String id) {
        this.id = id;
    }
    /**
     * Set ID.
     * @param id id
     */
    public void setId(int id) {
        setId("Picture#" + id);
    }
	// Duplicatable
	public Picture duplicate() {
		return new Picture(filename);
	}
	// Persistable:
    public String getId() {
        return id;
    }
	public String getType() {
		return "picture";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("filename", filename);
		res.put("id", id);
		return res;
	}
	public List<List<Persistable>> getChildren() {
		return new ArrayList<List<Persistable>>();
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		filename = attr.get("filename");
        id = attr.get("id");
	}
	public void setChildren(List<List<Persistable>> childs) {
	}
	// basic
	@Override
	public boolean equals(Object o) {
		if(o instanceof Picture) {
			Picture c = (Picture)o;
			if(id.equals(c.id)) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	@Override
	public int hashCode() {
		return id.hashCode();
	}
	@Override
	public String toString() {
	    return "(" + filename + "," + id + ")";
	}
}
