/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.awt.datatransfer.Transferable;

import javax.swing.JPanel;

import dk.vajhoej.model.Util;
import dk.vajhoej.model.core.Box;
import dk.vajhoej.model.core.BoxType;
import dk.vajhoej.model.core.Component;
import dk.vajhoej.model.core.ComponentStatus;
import dk.vajhoej.model.graphic.base.BoxGraphic;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class BoxGraphicWrapper contains the added GUI functionality for
 * a BoxGraphic instance. 
 */
public class BoxGraphicWrapper extends BoxGraphic implements CommonGraphicWrapper {
	private static final long serialVersionUID = 1L;
	private Box box;
	private static String label(Box b) {
		if((b.getVendor() + b.getModel()).length() > 0) {
			return b.getName() + " (" + b.getVendor() + " " + b.getModel() + ")";
		} else {
			return b.getName();
		}
	}
	private static String hardware(Box b) {
		StringBuilder sb = new StringBuilder();
		if(b.getCpu() > 0) {
			sb.append(Integer.toString(b.getCpu()));
			sb.append(" CPU cores");
		}
		if(b.getMem() > 0) {
			if(sb.length() > 0) sb.append(" ");
			sb.append(Util.intOrDoubleToString(b.getMem()));
			sb.append(" GB mem");
		}
		if(b.getDisk() > 0) {
			if(sb.length() > 0) sb.append(" ");
			sb.append(Util.intOrDoubleToString(b.getDisk()));
			sb.append(" TB disk");
		}
		return sb.length() > 0 ? sb.toString() : null;
	}
	/**
	 * Create instance of BoxGraphicWrapper.
	 * @param cfg all graphic configurations
	 * @param b box
	 */
	public BoxGraphicWrapper(AllGraphicConfig cfg, Box b) {
		super(cfg.getBoxConfig(b.getVendor()), b.getComponents().size(), label(b), hardware(b), b.getBoxtype());
		this.box = b;
		for(Component c : b.getComponents()) {
			add(new ComponentGraphicWrapper(cfg, c, cfg.isGrayPassive() && c.getStatus() == ComponentStatus.REMOTE_FAILOVER_PASSIVE));
		}
		GUISupport.setupDropAndDrag(this, this, cfg);
		GUISupport.setupContextMenus(this, this, cfg);
	}
	// CommonGraphicWrapper:
	public boolean isReadonly() {
		return readonly;
	}
	public boolean acceptItem(Object item) {
		if(item instanceof Component && (box.getBoxtype() == BoxType.SINGLE_SYS || box.getBoxtype() == BoxType.MULTI_SYS)) {
			return true;
		} else {
			return false;
		}
	}
	public void addItem(AllGraphicConfig cfg, Object item, Point p) {
		box.addComponent((Component)item);
		this.add(new ComponentGraphicWrapper(cfg, (Component)item));
		this.addChild();
		this.upAutoSize();
	}
	public void removeItem(CommonGraphicWrapper item) {
		box.removeComponent((Component)item.getRealObject());
		this.remove((JPanel)item);
		this.removeChild();
		this.upAutoSize();
	}
	public void updateState() {
		this.setLbl(label(box));
		this.setHw(hardware(box));
		this.setType(box.getBoxtype());
		this.upAutoSize();
	}
	public Box getRealObject() {
		return box;
	}
	public Transferable getTransferableObject() {
		return new ModelTransferable(box.duplicate());
	}
	public String getId() {
		return box.getName();
	}
	public boolean isRotating() {
		return false;
	}
	public void rotate(double angle, double hor, double ver) {
	}
    public double getAngle() {
        return 0.0;
    }
    public double getHor() {
        return 0.0;
    }
    public double getVer() {
        return 0.0;
    }
	public Object[] createNew() {
		return new Object[] { new Component() };
	}
}
