/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import dk.vajhoej.model.app.Data;
import dk.vajhoej.model.core.Box;
import dk.vajhoej.model.core.BoxPalette;
import dk.vajhoej.model.graphic.base.BoxPaletteGraphic;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class BoxPaletteGraphicWrapper contains the added GUI functionality for
 * a BoxPaletteGraphic instance. 
 */
public class BoxPaletteGraphicWrapper extends BoxPaletteGraphic implements CommonGraphicWrapper {
	private static final long serialVersionUID = 1L;
	/**
	 * Create instance of BoxPaletteGraphicWrapper
	 * @param cfg graphic configuration
	 * @param bp box palette
	 * @param data config data
	 */
	public BoxPaletteGraphicWrapper(AllGraphicConfig cfg, final BoxPalette bp, final Data data) {
		super(cfg.getPaletteConfig(), bp.getBoxes().size(), bp.getName(), data.getShowPalette().get(bp.getStore()));
		for(Box b : bp.getBoxes()) {
		    BoxGraphicWrapper bgw = new BoxGraphicWrapper(cfg, b);
		    bgw.setReadonly(true);
			add(bgw);
		}
		this.addMouseListener(new MouseAdapter() {
			public void mousePressed(final MouseEvent me) {
				if((me.getModifiers() & MouseEvent.BUTTON1_MASK) == 0) {
					JPopupMenu menu = new JPopupMenu();
					JMenuItem mitog = new JMenuItem(isShow() ? "Hide" : "Show");
					mitog.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
						    data.getShowPalette().put(bp.getStore(), !isShow());
							setShow(!isShow());
						}
					});
					menu.add(mitog);
                    JMenuItem miup = new JMenuItem("Move up");
                    miup.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int ix = data.getBoxPalette().indexOf(bp);
                            if(ix > 0) {
                                BoxPalette tmp = data.getBoxPalette().get(ix - 1);
                                data.getBoxPalette().set(ix - 1, bp);
                                data.getBoxPalette().set(ix, tmp);
                                BoxPaletteGraphicWrapper.this.swap(ix, ix - 1);
                            }
                        }
                    });
                    menu.add(miup);
                    JMenuItem midown = new JMenuItem("Move down");
                    midown.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int ix = data.getBoxPalette().indexOf(bp);
                            if(ix < data.getBoxPalette().size() - 1) {
                                BoxPalette tmp = data.getBoxPalette().get(ix + 1);
                                data.getBoxPalette().set(ix + 1, bp);
                                data.getBoxPalette().set(ix, tmp);
                                BoxPaletteGraphicWrapper.this.swap(ix, ix + 1);
                            }
                        }
                    });
                    menu.add(midown);
					menu.show(BoxPaletteGraphicWrapper.this, me.getX(), me.getY());
				}
			}
		});
		this.autoSize();
	}
	// CommonGraphicWrapper:
	public boolean isReadonly() {
		return readonly;
	}
	public boolean acceptItem(Object item) {
		return false;
	}
	public void addItem(AllGraphicConfig cfg, Object item, Point p) {
	}
	public void removeItem(CommonGraphicWrapper item) {
	}
	public void updateState() {
	}
	public Object getRealObject() {
		return null;
	}
	public Transferable getTransferableObject() {
		return null;
	}
	public String getId() {
		return "";
	}
	public boolean isRotating() {
		return false;
	}
	public void rotate(double angle, double hor, double ver) {
	}
    public double getAngle() {
        return 0.0;
    }
    public double getHor() {
        return 0.0;
    }
    public double getVer() {
        return 0.0;
    }
	public Object[] createNew() {
		return new Object[0];
	}
}
