/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;

/**
 * Class ClassDiagramPanel contains the necessary functionality to
 * display class diagram.
 */
public class ClassDiagramPanel extends JDialog {
	private static final long serialVersionUID = 1L;
	/**
	 * Create instance of ClassDiagramPanel.
	 * @param parent parent frame
	 */
	public ClassDiagramPanel(JDialog parent) {
		super(parent, "Class diagram", true);
		setLayout(new BorderLayout());
        try {
            BufferedImage img = ImageIO.read(this.getClass().getResourceAsStream("/VSMAClasses.gif"));
     		setPreferredSize(new Dimension(img.getWidth()+20, img.getHeight()+80));
            add(new ActualImage(img), BorderLayout.CENTER);
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(this, "Can not display class diagram");
        }
		JButton ext = new JButton("Exit");
		ext.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ClassDiagramPanel.this.dispose();
			}
		});
		add(ext, BorderLayout.SOUTH);
		pack();
	}
}

