/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.awt.datatransfer.Transferable;

import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Interface CommonGraphicWrapper describes the functionality required by
 * all the GUI wrapper classes.
 */
public interface CommonGraphicWrapper {
    /**
     * Is component readonly.
     * @return readonly
     */
	public boolean isReadonly();
	/**
	 * Will component accept item.
	 * @param item object to be added
	 * @return accept status
	 */
	public boolean acceptItem(Object item);
	/**
	 * Add item to component.
	 * @param cfg graphic configuration
	 * @param item object to be added
	 * @param p point to add it
	 */
	public void addItem(AllGraphicConfig cfg, Object item, Point p);
	/**
	 * Remove item from component.
	 * @param item object to be removed
	 */
	public void removeItem(CommonGraphicWrapper item);
	/**
	 * Update state of component.
	 */
	public void updateState();
	/**
	 * Get model object represented by component.
	 * @return model object
	 */
	public Object getRealObject();
	/**
	 * Get tranferable object.
	 * @return tranferable object
	 */
	public Transferable getTransferableObject();
	/**
	 * Is the component a rotating component.
	 * @return rotating
	 */
	public boolean isRotating();
	/**
	 * Rotate component.
	 * @param angle rotation angle
	 * @param hor horizontal move
	 * @param ver vertical move
	 */
	public void rotate(double angle, double hor, double ver);
	/**
	 * Get existing angle.
	 * @return angle
	 */
    public double getAngle();
    /**
     * Get existing horizontal move.
     * @return horizontal move
     */
    public double getHor();
    /**
     * Get existing vertical move.
     * @return vertical move.
     */
    public double getVer();
	/**
	 * Objects component can create.
	 * @return objects
	 */
	public Object[] createNew();
}
