/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import dk.vajhoej.model.app.Data;
import dk.vajhoej.model.core.Component;
import dk.vajhoej.model.core.ComponentPalette;
import dk.vajhoej.model.graphic.base.ComponentPaletteGraphic;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class ComponentPaletteGraphicWrapper contains the added GUI functionality for
 * a ComponentPaletteGraphic instance. 
 */
public class ComponentPaletteGraphicWrapper extends ComponentPaletteGraphic implements CommonGraphicWrapper {
	private static final long serialVersionUID = 1L;
	/**
	 * Create instance of ComponentPaletteGraphicWrapper.
	 * @param cfg all graphic configurations
	 * @param cp component palette
     * @param data config data
	 */
	public ComponentPaletteGraphicWrapper(AllGraphicConfig cfg, final ComponentPalette cp, final Data data) {
		super(cfg.getPaletteConfig(), cp.getComponents().size(), cp.getName(), data.getShowPalette().get(cp.getStore()));
		for(Component c : cp.getComponents()) {
		    ComponentGraphicWrapper cgw = new ComponentGraphicWrapper(cfg, c);
		    cgw.setReadonly(true);
			add(cgw);
		}
		this.addMouseListener(new MouseAdapter() {
			public void mousePressed(final MouseEvent me) {
				if((me.getModifiers() & MouseEvent.BUTTON1_MASK) == 0) {
					JPopupMenu menu = new JPopupMenu();
					JMenuItem mitog = new JMenuItem(isShow() ? "Hide" : "Show");
					mitog.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
                            data.getShowPalette().put(cp.getStore(), !isShow());
							setShow(!isShow());
						}
					});
					menu.add(mitog);
                    JMenuItem miup = new JMenuItem("Move up");
                    miup.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int ix = data.getComponentPalette().indexOf(cp);
                            if(ix > 0) {
                                ComponentPalette tmp = data.getComponentPalette().get(ix - 1);
                                data.getComponentPalette().set(ix - 1, cp);
                                data.getComponentPalette().set(ix, tmp);
                                ComponentPaletteGraphicWrapper.this.swap(ix, ix - 1);
                            }
                        }
                    });
                    menu.add(miup);
                    JMenuItem midown = new JMenuItem("Move down");
                    midown.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int ix = data.getComponentPalette().indexOf(cp);
                            if(ix < data.getComponentPalette().size() - 1) {
                                ComponentPalette tmp = data.getComponentPalette().get(ix + 1);
                                data.getComponentPalette().set(ix + 1, cp);
                                data.getComponentPalette().set(ix, tmp);
                                ComponentPaletteGraphicWrapper.this.swap(ix, ix + 1);
                            }
                        }
                    });
                    menu.add(midown);
					menu.show(ComponentPaletteGraphicWrapper.this, me.getX(), me.getY());
				}
			}
		});
		this.autoSize();
	}
	// CommonGraphicWrapper:
	public boolean isReadonly() {
		return readonly;
	}
	public boolean acceptItem(Object item) {
		return false;
	}
	public void addItem(AllGraphicConfig cfg, Object item, Point p) {
	}
	public void removeItem(CommonGraphicWrapper item) {
	}
	public void updateState() {
	}
	public Object getRealObject() {
		return null;
	}
	public Transferable getTransferableObject() {
		return null;
	}
	public String getId() {
		return "";
	}
	public boolean isRotating() {
		return false;
	}
	public void rotate(double angle, double hor, double ver) {
	}
    public double getAngle() {
        return 0.0;
    }
    public double getHor() {
        return 0.0;
    }
    public double getVer() {
        return 0.0;
    }
	public Object[] createNew() {
		return new Object[0];
	}
}
