/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.awt.datatransfer.Transferable;

import dk.vajhoej.model.core.Connection;
import dk.vajhoej.model.graphic.base.ConnectionGraphic;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class ConnectionGraphicWrapper contains the added GUI functionality for
 * a ConnectionGraphic instance. 
 */
public class ConnectionGraphicWrapper extends ConnectionGraphic implements CommonGraphicWrapper {
	private static final long serialVersionUID = 1L;
	private Connection con;
    /**
     * Create instance of ConnectionGraphicWrapper.
     * @param cfg all graphic configurations
     * @param c connection
     */
	public ConnectionGraphicWrapper(AllGraphicConfig cfg, Connection c) {
		super(cfg.getConnectionConfig(), c.getStack(), c.getConntype());
		this.con = c;
		GUISupport.setupDropAndDrag(this, this, cfg);
		GUISupport.setupContextMenus(this, this, cfg);
	}
	// CommonGraphicWrapper:
	public boolean isReadonly() {
		return readonly;
	}
	public boolean acceptItem(Object item) {
		return false;
	}
	public void addItem(AllGraphicConfig cfg, Object item, Point p) {
	}
	public void removeItem(CommonGraphicWrapper item) {
	}
	public void updateState() {
		this.setLbl(con.getStack());
		this.setConntype(con.getConntype());
		this.upAutoSize();
	}
	public Connection getRealObject() {
		return con;
	}
	public Transferable getTransferableObject() {
		return new ModelTransferable(con.duplicate());
	}
	public String getId() {
		return con.getStack();
	}
	public boolean isRotating() {
		return true;
	}
	public void rotate(double angle, double hor, double ver) {
		this.setAngle(angle);
        this.setHor(hor);
        this.setVer(ver);
		this.upAutoSize();
	}
	// getAngle, getHor and getVer already exists
	public Object[] createNew() {
		return new Object[0];
	}
}
