/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import dk.vajhoej.model.app.Data;
import dk.vajhoej.model.core.Connection;
import dk.vajhoej.model.core.ConnectionPalette;
import dk.vajhoej.model.graphic.base.ConnectionPaletteGraphic;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class ConnectionPaletteGraphicWrapper contains the added GUI functionality for
 * a ConnectionPaletteGraphic instance. 
 */
public class ConnectionPaletteGraphicWrapper extends ConnectionPaletteGraphic implements CommonGraphicWrapper {
	private static final long serialVersionUID = 1L;
    /**
     * Create instance of ConnectionPaletteGraphicWrapper.
     * @param cfg all graphic configurations
     * @param cp connection palette
     * @param data config data
     */
	public ConnectionPaletteGraphicWrapper(AllGraphicConfig cfg, final ConnectionPalette cp, final Data data) {
		super(cfg.getPaletteConfig(), cp.getConnections().size(), cp.getName(), data.getShowPalette().get(cp.getStore()));
		for(Connection b : cp.getConnections()) {
		    ConnectionGraphicWrapper cgw = new ConnectionGraphicWrapper(cfg, b);
		    cgw.setReadonly(readonly);
			add(cgw);
		}
		this.addMouseListener(new MouseAdapter() {
			public void mousePressed(final MouseEvent me) {
				if((me.getModifiers() & MouseEvent.BUTTON1_MASK) == 0) {
					JPopupMenu menu = new JPopupMenu();
					JMenuItem mitog = new JMenuItem(isShow() ? "Hide" : "Show");
					mitog.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
                            data.getShowPalette().put(cp.getStore(), !isShow());
							setShow(!isShow());
						}
					});
					menu.add(mitog);
                    JMenuItem miup = new JMenuItem("Move left");
                    miup.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int ix = data.getConnectionPalette().indexOf(cp);
                            if(ix > 0) {
                                ConnectionPalette tmp = data.getConnectionPalette().get(ix - 1);
                                data.getConnectionPalette().set(ix - 1, cp);
                                data.getConnectionPalette().set(ix, tmp);
                                ConnectionPaletteGraphicWrapper.this.swap(ix, ix - 1);
                            }
                        }
                    });
                    menu.add(miup);
                    JMenuItem midown = new JMenuItem("Move right");
                    midown.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int ix = data.getConnectionPalette().indexOf(cp);
                            if(ix < data.getConnectionPalette().size() - 1) {
                                ConnectionPalette tmp = data.getConnectionPalette().get(ix + 1);
                                data.getConnectionPalette().set(ix + 1, cp);
                                data.getConnectionPalette().set(ix, tmp);
                                ConnectionPaletteGraphicWrapper.this.swap(ix, ix + 1);
                            }
                        }
                    });
                    menu.add(midown);
					menu.show(ConnectionPaletteGraphicWrapper.this, me.getX(), me.getY());
				}
			}
		});
		this.autoSize();
	}
	// CommonGraphicWrapper:
	public boolean isReadonly() {
		return readonly;
	}
	public boolean acceptItem(Object item) {
		return false;
	}
	public void addItem(AllGraphicConfig cfg, Object item, Point p) {
	}
	public void removeItem(CommonGraphicWrapper item) {
	}
	public void updateState() {
	}
	public Object getRealObject() {
		return null;
	}
	public Transferable getTransferableObject() {
		return null;
	}
	public String getId() {
		return "";
	}
	public boolean isRotating() {
		return false;
	}
	public void rotate(double angle, double hor, double ver) {
	}
    public double getAngle() {
        return 0.0;
    }
    public double getHor() {
        return 0.0;
    }
    public double getVer() {
        return 0.0;
    }
	public Object[] createNew() {
		return new Object[0];
	}
}
