/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.awt.datatransfer.Transferable;

import dk.vajhoej.model.graphic.base.EmptyGraphic;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class EmptyGraphicWrapper contains the added GUI functionality for
 * a EmptyGraphic instance. 
 */
public class EmptyGraphicWrapper extends EmptyGraphic implements CommonGraphicWrapper {
	private static final long serialVersionUID = 1L;
	/**
	 * Create instance of EmptyGraphicWrapper.
	 * @param cfg all graphic configurations
	 */
	public EmptyGraphicWrapper(AllGraphicConfig cfg) {
		super(cfg.getModelConfig());
	}
	// CommonGraphicWrapper:
	public boolean isReadonly() {
		return readonly;
	}
	public boolean acceptItem(Object item) {
		return false;
	}
	public void addItem(AllGraphicConfig cfg, Object item, Point p) {
	}
	public void removeItem(CommonGraphicWrapper item) {
	}
	public void updateState() {
	}
	public Object getRealObject() {
		return null;
	}
	public Transferable getTransferableObject() {
		return null;
	}
	public String getId() {
		return "";
	}
	public boolean isRotating() {
		return false;
	}
	public void rotate(double angle, double hor, double ver) {
	}
    public double getAngle() {
        return 0.0;
    }
    public double getHor() {
        return 0.0;
    }
    public double getVer() {
        return 0.0;
    }
	public Object[] createNew() {
		return new Object[0];
	}
}
