/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

import dk.vajhoej.model.graphic.config.GraphicConfig;

/**
 * Class ObjectEditPanel contains the necessary functionality to
 * edit a GraphicConfig object.
 */
public class GraphicEditPanel extends JDialog {
	private static final long serialVersionUID = 1L;
	/**
	 * Create instance of GraphicEditPanel.
	 * @param parent parent frame
	 * @param cfg GraphicConfig to edit
	 */
	public GraphicEditPanel(JFrame parent, final GraphicConfig cfg) {
		super(parent, "Edit graphics", true);
		final GraphicConfig cfg2 = (GraphicConfig)cfg.clone();
		setLayout(new BorderLayout());
		final JLabel show = new JLabel();
		JPanel show2 = new JPanel();
        show2.setLayout(new GridLayout(1, 1));
        show2.setBorder(new EmptyBorder(5, 5, 5, 5));
		show2.add(show);
		add(show2, BorderLayout.NORTH);
        updateShow(show, cfg2);
		JPanel all = new JPanel();
		all.setLayout(new GridLayout(4, 3, 10, 10));
        all.setBorder(new EmptyBorder(5, 5, 5, 5));
		all.add(new JLabel("Line thickness:"));
		all.add(new JLabel("Font name:"));
		all.add(new JLabel("Font size:"));
		JComboBox thckcb = new JComboBox(new Integer[] { 1, 2, 3, 4, 5, 6 });
		thckcb.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				cfg2.setThickness((Integer)e.getItem());
		        updateShow(show, cfg2);
			}
		});
		thckcb.setSelectedItem(cfg2.getThickness());
		all.add(thckcb);
		JComboBox fntncb = new JComboBox(new String[] { "Arial", "Book Antiqua", "Century Gothic", "Courier New", "Garamond", "Palatino Linotype", "Times New Roman",
				                                        "Arial Bold", "Book Antiqua Bold", "Century Gothic Bold", "Courier New Bold", "Garamond Bold", "Palatino Linotype Bold", "Times New Roman Bold"});
		fntncb.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				cfg2.setFont(Font.decode(e.getItem() + "-" + cfg2.getFont().getSize()));
		        updateShow(show, cfg2);
			}
		});
		fntncb.setSelectedItem(cfg2.getFont().getFontName());
		all.add(fntncb);
		JComboBox fntscb = new JComboBox(new Integer[] { 6, 8, 10, 12, 14, 16, 18, 24, 36, 48, 72 });
		fntscb.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				cfg2.setFont(Font.decode(cfg2.getFont().getFontName() + "-" + e.getItem()));
		        updateShow(show, cfg2);
			}
		});
		fntscb.setSelectedItem(cfg2.getFont().getSize());
		all.add(fntscb);
		JButton txtbtn = new JButton("Text color");
		txtbtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
                Color c = JColorChooser.showDialog(GraphicEditPanel.this, "Choose text color", cfg2.getTextColor());
                if(c != null) {
                    cfg2.setTextColor(c);
                    updateShow(show, cfg2);
                }
			}
		});
		all.add(txtbtn);
		JButton linbtn = new JButton("Line color");
		linbtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
                Color c = JColorChooser.showDialog(GraphicEditPanel.this, "Choose line color", cfg2.getLineColor());
                if(c != null) {
                    cfg2.setLineColor(c);
                    updateShow(show, cfg2);
                }
			}
		});
		all.add(linbtn);
		JButton filbtn = new JButton("Fill color");
		filbtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
                Color c = JColorChooser.showDialog(GraphicEditPanel.this, "Choose fill color", cfg2.getFillColor());
                if(c != null) {
                    cfg2.setFillColor(c);
                    updateShow(show, cfg2);
                }
			}
		});
		all.add(filbtn);
		all.add(new JLabel("Round corners:"));
		JCheckBox jcb = new JCheckBox();
		jcb.setSelected(cfg2.isRound());
		jcb.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                cfg2.setRound(!cfg2.isRound());
                updateShow(show, cfg2);
            }
		});
		all.add(jcb);
		add(all, BorderLayout.CENTER);
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cfg.setThickness(cfg2.getThickness());
				cfg.setFont(cfg2.getFont());
				cfg.setTextColor(cfg2.getTextColor());
				cfg.setLineColor(cfg2.getLineColor());
				cfg.setFillColor(cfg2.getFillColor());
				cfg.setRound(cfg2.isRound());
				GraphicEditPanel.this.dispose();
			}
		});
		JButton can = new JButton("Cancel");
		can.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				GraphicEditPanel.this.dispose();
			}
		});
		JPanel btns = new JPanel();
		btns.setLayout(new GridLayout(1, 2, 10, 10));
		btns.setBorder(new EmptyBorder(5, 5, 5, 5));
		btns.add(ok);
		btns.add(can);
		add(btns, BorderLayout.SOUTH);
		pack();
	}
	private void updateShow(JLabel show, GraphicConfig cfg) {
	    show.setBorder(new LineBorder(cfg.getLineColor(), cfg.getThickness()));
	    show.setFont(cfg.getFont());
	    show.setForeground(cfg.getTextColor());
	    show.getParent().setBackground(cfg.getFillColor());
	    show.setText("This is a test");
	}
}
