/*
 * Copyright 2011 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import dk.vajhoej.model.app.Application;
import dk.vajhoej.model.app.Data;
import dk.vajhoej.model.app.DisplayFilter;

/**
 * Class MultiFilterPanel applies multiple filters to model.
 */
public class MultiFilterPanel extends JDialog {
	private static final long serialVersionUID = 1L;
	/**
	 * Create instance of GraphicEditPanel.
	 * @param parent parent frame
	 * @param allfilter all display filters
	 * @param data data
	 */
	public MultiFilterPanel(JFrame parent, final DisplayFilter[] allfilter, final Data data) {
		super(parent, "Multi filter", false);
		setLayout(new BorderLayout());
		JPanel all = new JPanel();
		all.setLayout(new GridLayout(allfilter.length, 2, 10, 10));
        all.setBorder(new EmptyBorder(5, 5, 5, 5));
        final JCheckBox[] jcb = new JCheckBox[allfilter.length];
        for(int i = 0; i < allfilter.length; i++) {
            all.add(new JLabel(allfilter[i].getName()));
            jcb[i] = new JCheckBox();
            all.add(jcb[i]);
        }
		add(all, BorderLayout.CENTER);
		JButton show = new JButton("Show");
		show.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    Data d = data;
		        for(int i = 0; i < allfilter.length; i++) {
		            if(jcb[i].isSelected()) {
	                    d = allfilter[i].filter(d);
		            }
		        }
                d.setReadonly(true);
                Application app2 = new Application(d, false);
                app2.setVisible(true);
			}
		});
		JButton clos = new JButton("Close");
		clos.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				MultiFilterPanel.this.dispose();
			}
		});
		JPanel btns = new JPanel();
		btns.setLayout(new GridLayout(1, 2, 10, 10));
		btns.setBorder(new EmptyBorder(5, 5, 5, 5));
		btns.add(show);
		btns.add(clos);
		add(btns, BorderLayout.SOUTH);
		pack();
	}
}
