/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.filechooser.FileSystemView;

import dk.vajhoej.model.FileBrowse;
import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.Util;

/**
 * Class ObjectEditPanel contains the necessary functionality to
 * edit any Persistable object.
 */
public class ObjectEditPanel extends JDialog {
	private static final long serialVersionUID = 1L;
	/**
	 * Create instance of ObjectEditPanel.
	 * @param parent parent frame
	 * @param p persistable object to edit
	 * @throws NoSuchFieldException 
	 */
	public ObjectEditPanel(JFrame parent, final Persistable p) {
		super(parent, "Edit " + p.getType(), true);
		setLayout(new BorderLayout());
		final NameValuePairList attr = p.getAttributes();
		final JComponent[] jc = new JComponent[attr.size()];
		JPanel alltf = new JPanel();
		alltf.setLayout(new GridLayout(attr.size() - 1, 2, 10, 10));
		alltf.setBorder(new EmptyBorder(5, 5, 5, 5));
		String id = null;
		for(int i = 0; i < attr.size(); i++) {
		    if(!attr.getName(i).equals("id")) {
	            try {
	                alltf.add(new JLabel(Util.toCap(attr.getName(i))));
	                Class<?> typ = p.getClass().getMethod("get" + Util.toCap(attr.getName(i)), new Class[0]).getReturnType();
	                if(typ.isEnum()) {
	                    Object[] evals = typ.getEnumConstants();
	                    JComboBox cb = new JComboBox(evals);
	                    int ix = -1;
	                    for(int j = 0; j < evals.length; j++) {
	                        if(evals[j].toString().equals(attr.getValue(i))) {
	                            ix = j;
	                        }
	                    }
	                    cb.setSelectedIndex(ix);
	                    jc[i] = cb;
		                alltf.add(cb);
	                } else if(!attr.getName(i).equals("version") &&
	                		  p.getClass().getDeclaredField(attr.getName(i)).isAnnotationPresent(FileBrowse.class)) {
	                	final String dir = p.getClass().getDeclaredField(attr.getName(i)).getAnnotation(FileBrowse.class).directory();
	                    final JTextField tf = new JTextField();
	                    tf.setText(attr.getValue(i));
	                    jc[i] = tf;
	                    JButton btn = new JButton("Browse");
	                    btn.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent evt) {
						        JFileChooser fc = new JFileChooser(dir, new SingleDirFileSystemView(dir));
						        if(fc.showOpenDialog(ObjectEditPanel.this) == JFileChooser.APPROVE_OPTION) {
						            tf.setText(dir + "/" + fc.getSelectedFile().getName());
						        }
							}
	                    });
	                    JPanel tfbtn = new JPanel();
	            		tfbtn.setLayout(new GridLayout(1, 2, 10, 10));
	            		tfbtn.setBorder(new EmptyBorder(5, 5, 5, 5));
	            		tfbtn.add(tf);
	            		tfbtn.add(btn);
	            		alltf.add(tfbtn);
	                } else {
	                    JTextField tf = new JTextField();
	                    tf.setText(attr.getValue(i));
	                    jc[i] = tf;
		                alltf.add(tf);
	                }
	            } catch (IllegalArgumentException ex) {
	                ex.printStackTrace();
	            } catch (SecurityException ex) {
	                ex.printStackTrace();
	            } catch (NoSuchMethodException ex) {
	                ex.printStackTrace();
	            } catch (NoSuchFieldException ex) {
					ex.printStackTrace();
				}
		    } else {
		        id = attr.getValue(i);
		    }
		}
		final String id2 = id;
		add(alltf, BorderLayout.CENTER);
		JButton ok = new JButton("OK");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				NameValuePairList res = new NameValuePairList();
				for(int i = 0; i < attr.size(); i++) {
		            if(!attr.getName(i).equals("id")) {
	                    if(jc[i] instanceof JComboBox) {
	                        res.put(attr.getName(i), ((JComboBox)jc[i]).getSelectedItem().toString());
	                    } else {
	                        res.put(attr.getName(i), ((JTextField)jc[i]).getText());
	                    }
		            }
				}
				if(id2 != null) {
				    res.put("id", id2);
				}
				try {
					p.setAttributes(res);
	                ObjectEditPanel.this.dispose();
				} catch (ModelException ex) {
				    JOptionPane.showMessageDialog(ObjectEditPanel.this, ex.getMessage());
				}
			}
		});
		JButton can = new JButton("Cancel");
		can.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ObjectEditPanel.this.dispose();
			}
		});
		JPanel btns = new JPanel();
		btns.setLayout(new GridLayout(1, 2, 10, 10));
		btns.setBorder(new EmptyBorder(5, 5, 5, 5));
		btns.add(ok);
		btns.add(can);
		add(btns, BorderLayout.SOUTH);
		pack();
	}
}

class SingleDirFileSystemView extends FileSystemView {
	private File dir;
	public SingleDirFileSystemView(String dirnam) {
		dir = new File(dirnam);
	}
	public File createNewFolder(File newdir) {    
		return null;
    }
	@Override
    public File[] getRoots() {
        return new File[] { dir };
    }
    @Override
    public boolean isRoot(File file)
    {
    	try {
			return dir.getCanonicalPath().equals(file.getCanonicalPath());
		} catch (IOException e) {
			return false;
		}
    }
    @Override
    public File getHomeDirectory() {
        return dir;
    }
}
