/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.io.StringReader;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;

import bsh.EvalError;
import bsh.Interpreter;

import dk.vajhoej.model.app.Data;

/**
 * Class ScriptPanel contains the necessary functionality to
 * execute small scripts.
 */
public class ScriptPanel extends JDialog {
	private static final long serialVersionUID = 1L;
	/**
	 * Create instance of ScriptPanel.
	 * @param parent parent frame
	 * @param data all the data
	 */
	public ScriptPanel(JFrame parent, final Data data) {
		super(parent, "Script execute", true);
		setLayout(new BorderLayout());
		final JTextArea src = new JTextArea();
        JButton boxes = new JButton("Iterate over boxes");
        boxes.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                src.setText("for(Box b : model.getBoxes()) {\r\n" +
                            "   System.out.println(b.getName());\r\n" +
                            "}\r\n");
            }
        });
        JButton comps = new JButton("Iterate over components");
        comps.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                src.setText("for(Box b : model.getBoxes()) {\r\n" +
                            "   for(Component c : b.getComponents()) {\r\n" +
                            "       System.out.println(c.getName());\r\n" +
                            "   }\r\n" +
                            "}\r\n");
            }
        });
        JButton conns = new JButton("Iterate over connections");
        conns.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                src.setText("for(Connection c : model.getConnections()) {\r\n" +
                            "   System.out.println(c.getStack());\r\n" +
                            "}\r\n");
            }
        });
        JButton grps = new JButton("Iterate over graphics");
        grps.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                src.setText("for(String s : graphic.getBoxes()) {\r\n" +
                            "   System.out.println(s + ' ' + graphic.getBoxConfig(s).getFillColor());\r\n" +
                            "}\r\n" +
                            "for(String s : graphic.getComponents()) {\r\n" +
                            "   System.out.println(s + ' ' + graphic.getComponentConfig(s).getFillColor());\r\n" +
                            "}\r\n");
            }
        });
        JButton dia = new JButton("Show class diagram");
        dia.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JDialog clsdia = new ClassDiagramPanel(ScriptPanel.this);
                clsdia.setVisible(true);
            }
        });
        JPanel macwnd = new JPanel();
        macwnd.setLayout(new GridLayout(1, 5, 10, 10));
        macwnd.setBorder(new EmptyBorder(5, 5, 5, 5));
        macwnd.add(boxes);
        macwnd.add(comps);
        macwnd.add(conns);
        macwnd.add(grps);
        macwnd.add(dia);
        add(macwnd, BorderLayout.NORTH);
		JPanel wnds = new JPanel();
		wnds.setLayout(new GridLayout(3, 1, 10, 10));
        wnds.setBorder(new EmptyBorder(5, 5, 5, 5));
        wnds.setPreferredSize(new Dimension(400, 400));
        JPanel srcwnd = new JPanel();
        srcwnd.setLayout(new BorderLayout());
        srcwnd.add(new JLabel("Source code"), BorderLayout.NORTH);
        srcwnd.add(new JScrollPane(src), BorderLayout.CENTER);
        wnds.add(srcwnd);
        final JTextArea out = new JTextArea();
        JPanel outwnd = new JPanel();
        outwnd.setLayout(new BorderLayout());
        outwnd.add(new JLabel("Output"), BorderLayout.NORTH);
        outwnd.add(new JScrollPane(out), BorderLayout.CENTER);
        wnds.add(outwnd);
        final JTextArea err = new JTextArea();
        JPanel errwnd = new JPanel();
        errwnd.setLayout(new BorderLayout());
        errwnd.add(new JLabel("Errors"), BorderLayout.NORTH);
        errwnd.add(new JScrollPane(err), BorderLayout.CENTER);
        wnds.add(errwnd);
		add(wnds, BorderLayout.CENTER);
		JButton ok = new JButton("Execute");
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    try {
                    ByteArrayOutputStream baos1 = new ByteArrayOutputStream();
                    PrintStream ps1 = new PrintStream(baos1);
                    ByteArrayOutputStream baos2 = new ByteArrayOutputStream();
                    PrintStream ps2 = new PrintStream(baos2);
                    PrintStream savout = System.out;
                    System.setOut(ps1);
                    PrintStream saverr = System.err;
                    System.setErr(ps2);
                    Interpreter i = new Interpreter(new StringReader(src.getText()), ps1, ps2, false);
	                i.eval("import dk.vajhoej.model.core.*");
                    i.set("model", data.getModel().getReal());
                    i.set("graphic", data.getConfig());
                    i.run();
                    ps1.flush();
                    ps2.flush();
                    out.setText(baos1.toString());
                    err.setText(baos2.toString());
                    System.setOut(savout);
                    System.setErr(saverr);
                } catch (EvalError ex) {
                    ex.printStackTrace();
                }
			}
		});
		JButton can = new JButton("Exit");
		can.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ScriptPanel.this.dispose();
			}
		});
		JPanel btns = new JPanel();
		btns.setLayout(new GridLayout(1, 2, 10, 10));
        btns.setBorder(new EmptyBorder(5, 5, 5, 5));
		btns.add(ok);
		btns.add(can);
		add(btns, BorderLayout.SOUTH);
		pack();
	}
}
