/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.base;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;

import dk.vajhoej.model.core.ComponentType;
import dk.vajhoej.model.graphic.config.GraphicConfig;

/**
 * Class ComponentGraphic is the graphic object for software component.
 */
public class ComponentGraphic extends BaseGraphic {
	private static final long serialVersionUID = 1L;
	private String lbl;
	private String hw;
	private ComponentType type;
	/**
     * Create instance of ComponentGraphic with dynamic size.
     * @param cfg graphic configuration
	 * @param nchild number children
	 * @param lbl label
	 * @param hw hardware
	 * @param type component type
	 */
	public ComponentGraphic(GraphicConfig cfg, int nchild, String lbl, String hw, ComponentType type) {
		super(cfg, nchild, GraphicLayout.SQUARE, hw != null ? 2 : 1, type==ComponentType.APPUML);
		this.lbl = lbl;
		this.hw = hw;
		this.type = type;
	}
	/**
     * Create instance of ComponentGraphic with dynamic size.
     * @param cfg graphic configuration
	 * @param w width
	 * @param h height
     * @param nchild number children
     * @param lbl label
     * @param hw hardware
     * @param type component type
	 */
	public ComponentGraphic(GraphicConfig cfg, int w, int h, int nchild, String lbl, String hw, ComponentType type) {
		super(cfg, w, h, nchild, GraphicLayout.SQUARE, hw != null ? 2 : 1, type==ComponentType.APPUML);
		this.lbl = lbl;
		this.hw = hw;
		this.type = type;
	}
	/**
	 * Set label.
	 * @param lbl label
	 */
	protected void setLbl(String lbl) {
		this.lbl = lbl;
	}
	/**
	 * Set hardware.
	 * @param hw hardware
	 */
	protected void setHw(String hw) {
		this.hw = hw;
		this.setBorder(hw != null ? 2 : 1, type==ComponentType.APPUML);
	}
	/**
	 * Set component type.
	 * @param type component type
	 */
	protected void setType(ComponentType type) {
		this.type = type;
	}
	@Override
	protected Dimension calcSize() {
	    Dimension res;
		if(hw != null) {
			res = addTwoLabels(super.calcSize(), lbl,  hw);
		} else {
			res = addLabel(super.calcSize(), lbl);
		}
		if(type == ComponentType.APPUML) {
		    res.width += 8*cfg.getMargin();
		    res.height = Math.max(res.height, 8*cfg.getMargin());
		}
		return res;
	}
	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D)g;
		switch(type) {
            case APP:
                drawRectangle(g2);
                drawTopString(lbl, g2);
                break;
            case APPUML:
                drawUML(g2);
                drawTopString(lbl, g2);
                break;
			case DB:
				drawBarrel(g2);
				drawMiddleString(lbl, g2);
				break;
			case VM:
				drawRectangle(g2);
				drawTopString(lbl, g2);
				if(hw != null) {
					drawBelowTopString(hw, g2);
				}
				break;
		}
    }
}
