/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.base;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;

import dk.vajhoej.model.graphic.config.GraphicConfig;

/**
 * Class EmptyGraphic is the graphic object for nothing.
 * This class is only used for technical reasons.
 */
public class EmptyGraphic extends BaseGraphic {
	private static final long serialVersionUID = 1L;
	/**
     * Create instance of EmptyGraphic with dynamic size.
     * @param cfg graphic configuration
	 */
	public EmptyGraphic(GraphicConfig cfg) {
		super(cfg, 0, GraphicLayout.SQUARE, 0, false);
	}
	/**
     * Create instance of EmptyGraphic with dynamic size.
     * @param cfg graphic configuration
	 * @param w width
	 * @param h height
	 */
	public EmptyGraphic(GraphicConfig cfg, int w, int h) {
		super(cfg, w, h, 0, GraphicLayout.SQUARE, 0, false);
	}
	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D)g;
		g2.setColor(Color.WHITE);
		g2.fillRect(0, 0, getWidth(), getHeight());
    }
}
