/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.base;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;

import dk.vajhoej.model.graphic.config.GraphicConfig;

/**
 * Class ModelGraphic is the graphic object for an architectural model.
 */
public class ModelGraphic extends BaseGraphic {
	private static final long serialVersionUID = 1L;
	private String lbl;
	/**
     * Create instance of ModelGraphic with dynamic size.
     * @param cfg graphic configuration
     * @param nchild number children
     * @param lbl label
     * @param style layout style
	 */
	public ModelGraphic(GraphicConfig cfg, int nchild, String lbl, GraphicLayout style) {
		super(cfg, nchild, style, 1, false);
		this.lbl = lbl;
	}
	/**
     * Create instance of ModelGraphic with fixed size.
     * @param cfg graphic configuration
	 * @param w width
	 * @param h height
     * @param nchild number children
     * @param lbl label
     * @param style layout style
	 */
	public ModelGraphic(GraphicConfig cfg, int w, int h, int nchild, String lbl, GraphicLayout style) {
		super(cfg, w, h, nchild, style, 1, false);
		this.lbl = lbl;
	}
	public void setLbl(String lbl) {
		this.lbl = lbl;
	}
	@Override
	protected Dimension calcSize() {
		return addLabel(super.calcSize(), lbl);
	}
	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D)g;
		g2.setColor(Color.WHITE);
		g2.fillRect(0, 0, getWidth(), getHeight());
		drawTopString(lbl, g2);
    }
}
