/*
 * Copyright 2011 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.base;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.graphic.config.GraphicConfig;

/**
 * Class PictureGraphic is the graphic object for a picture with builtin functionality.
 * This class is only used for technical reasons.
 */
public class PictureGraphic extends BaseGraphic {
	private static final long serialVersionUID = 1L;
	private Image img = null;
	/**
     * Create instance of PictureGraphic with dynamic size.
     * @param cfg graphic configuration
     * @param picfnm picture filename
	 */
	public PictureGraphic(GraphicConfig cfg, String picfnm) {
		super(cfg, 0, GraphicLayout.SQUARE, 0, false);
        try {
        	if(!picfnm.equals("")) {
    			img = ImageIO.read(new File(picfnm));
        	}
		} catch (IOException e) {
			throw new ModelRuntimeException("Error loading picture from " + picfnm, e);
		}
	}
	/**
     * Create instance of PictureGraphic with dynamic size.
     * @param cfg graphic configuration
	 * @param w width
	 * @param h height
     * @param picfnm picture filename
	 */
	public PictureGraphic(GraphicConfig cfg, int w, int h, String picfnm) {
		super(cfg, w, h, 0, GraphicLayout.SQUARE, 0, false);
        try {
        	if(!picfnm.equals("")) {
    			img = ImageIO.read(new File(picfnm));
        	}
		} catch (IOException e) {
			throw new ModelRuntimeException("Error loading picture from " + picfnm, e);
		}
	}
	/**
	 * Set picture filename.
	 * @param picfnm picture filename
	 */
	public void setFilename(String picfnm) {
        try {
        	if(!picfnm.equals("")) {
    			img = ImageIO.read(new File(picfnm));
        	}
		} catch (IOException e) {
			throw new ModelRuntimeException("Error loading picture from " + picfnm, e);
		}
	}
	@Override
	protected Dimension calcSize() {
		return img != null ? new Dimension(img.getWidth(this), img.getHeight(this)) : new Dimension(0, 0);
	}
	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D)g;
		if(img != null) {
			g2.drawImage(img.getScaledInstance(getWidth(), getHeight(), Image.SCALE_DEFAULT), 0, 0, this);
		}
    }
}
