/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.config;

import java.util.List;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;

/**
 * Class ExtGraphicConfig extends class GraphicConfig with a key.
 */
public class ExtGraphicConfig implements Persistable {
	private String key;
	private GraphicConfig real;
	/**
     * Create instance of ExtGraphicConfig.
	 */
	public ExtGraphicConfig() {
		this(null, new GraphicConfig());
	}
	/**
     * Create instance of ExtGraphicConfig.
	 * @param key key
	 * @param real GraphicConfig
	 */
	public ExtGraphicConfig(String key, GraphicConfig real) {
		this.key = key;
		this.real = real;
	}
	/**
	 * Get key.
	 * @return key
	 */
	public String getKey() {
		return key;
	}
	/**
	 * Get GraphicConfig.
	 * @return GraphicConfig.
	 */
	public GraphicConfig getConfig() {
		return real;
	}
	// Persistable
	public String getId() {
		return key;
	}
	public String getType() {
		return "extGraphicConfig";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = real.getAttributes();
		res.put("key", key);
		return res;
	}
	public List<List<Persistable>> getChildren() {
		return real.getChildren();
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		real.setAttributes(attr);
		key = attr.get("key");
	}
	public void setChildren(List<List<Persistable>> childs)	throws ModelException {
		real.setChildren(childs);
	}
	// basic
	@Override
	public String toString(){
	    return "(" + key + "," + real + ")";
	}
}
