/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.persist;

import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ls.LSException;
import org.xml.sax.SAXException;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePair;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.Persister;
import dk.vajhoej.model.SeparatePersistable;
import dk.vajhoej.model.Util;

/**
 * Class XmlPersister is a Persister implementation that uses
 * XML files as storage.
 * <BR>
 * Note that it uses Xerces specific functionality.
 */
public class XmlPersister implements Persister {
    static {
        System.setProperty("javax.xml.parsers.DocumentBuilderFactory", "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
    }
    public String getParams() {
        return "";
    }
    public void setParams(String params) {
        // ignore
    }
	private static void toDoc(Persistable p, Document doc, Node parent) {
		Element elm = doc.createElement(p.getType());
		elm.setAttribute("class", p.getClass().getName());
		NameValuePairList attr = p.getAttributes();
		for(NameValuePair nvp : attr) {
			elm.setAttribute(nvp.getName(), nvp.getValue());
		}
		for(List<Persistable> lst : p.getChildren()) {
			Element col = doc.createElement("list");
			for(Persistable p2 : lst) {
				toDoc(p2, doc, col);
			}
			elm.appendChild(col);
		}
		parent.appendChild(elm);
	}
	@SuppressWarnings("deprecation")
    public void saveSmart(SeparatePersistable p, String target, String parent) throws ModelException {
        try {
            String realtarget = target.replace("*", parent);
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document doc = db.newDocument();
			toDoc(p, doc, doc);
			OutputStream os = new FileOutputStream(realtarget);
			org.apache.xml.serialize.OutputFormat fmt = new org.apache.xml.serialize.OutputFormat();
            fmt.setIndenting(true);
            fmt.setLineWidth(10000);
            org.apache.xml.serialize.XMLSerializer ser = new org.apache.xml.serialize.XMLSerializer(os, fmt);
            ser.serialize(doc);
			os.close();
		} catch (IOException e) {
			throw new ModelException("Exception creating XML:", e);
		} catch (DOMException e) {
			throw new ModelException("Exception creating XML:", e);
		} catch (LSException e) {
			throw new ModelException("Exception creating XML:", e);
		} catch (ParserConfigurationException e) {
			throw new ModelException("Exception creating XML:", e);
		}
	}
    public void saveSmart(SeparatePersistable p, String parent) throws ModelException {
		saveSmart(p, p.getStore(), parent);
	}
    public void save(SeparatePersistable p, String target) throws ModelException {
        saveSmart(p, target, "");
    }
    public void save(SeparatePersistable p) throws ModelException {
        saveSmart(p, "");
    }
	@SuppressWarnings("unchecked")
	private static Persistable fromDoc(Element data) throws ModelException {
		try {
			String clznam = data.getAttribute("class");
			Class<Persistable> clz = (Class<Persistable>)Class.forName(clznam);
			Persistable p = clz.newInstance();
			NameValuePairList attr = new NameValuePairList();
			NamedNodeMap dataattr = data.getAttributes();
			for(int i = 0; i < dataattr.getLength(); i++) {
				Node a = dataattr.item(i);
				attr.put(a.getNodeName(), a.getNodeValue());
			}
			p.setAttributes(attr);
			List<List<Persistable>> childs = new ArrayList<List<Persistable>>();
			NodeList lists = data.getChildNodes();
			for(int j = 0; j < lists.getLength(); j++) {
				if(lists.item(j).getNodeType() == Node.ELEMENT_NODE) {
					List<Persistable> list = new ArrayList<Persistable>();
					NodeList datachilds = lists.item(j).getChildNodes();
					for(int i = 0; i < datachilds.getLength(); i++) {
						if(datachilds.item(i).getNodeType() == Node.ELEMENT_NODE) {
							list.add(fromDoc((Element)datachilds.item(i)));
						}
					}
					childs.add(list);
				}
			}
			p.setChildren(childs);
			return p;
		} catch (ClassNotFoundException e) {
			throw new ModelException("Exception loading model class:", e);
		} catch (InstantiationException e) {
			throw new ModelException("Exception loading model class:", e);
		} catch (IllegalAccessException e) {
			throw new ModelException("Exception loading model class:", e);
		}
	}
    public SeparatePersistable loadSmart(String source, Class<? extends SeparatePersistable> clz, String parent) throws ModelException {
        try {
            String realsource = source.replace("*", parent);
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			InputStream is = new FileInputStream(realsource);
			Document doc = db.parse(is);
			is.close();
			SeparatePersistable res = (SeparatePersistable)fromDoc(doc.getDocumentElement());
			res.setStore(realsource);
			if(!clz.isAssignableFrom(res.getClass())) {
			    throw new ModelException("Unexpected type found in " + source);
			}
			return res;
		} catch (IOException e) {
			throw new ModelException("Exception parsing XML:", e);
		} catch (ParserConfigurationException e) {
			throw new ModelException("Exception parsing XML:", e);
		} catch (SAXException e) {
			throw new ModelException("Exception parsing XML:", e);
		}
	}
    public SeparatePersistable load(String source, Class<? extends SeparatePersistable> clz) throws ModelException {
        return loadSmart(source, clz, "");
    }
    private JFileChooser createFileChooser(String dir, final String ext, String def) {
        JFileChooser res = new JFileChooser(dir);
        if(def != null) {
            res.setSelectedFile(new File(Util.combinePath(dir, def)));
        }
        res.setFileFilter(new FileFilter() {
            @Override
            public boolean accept(File f) {
                return f.isDirectory() || f.getName().endsWith("." + ext);
            }
            @Override
            public String getDescription() {
                return ext.toUpperCase() + " files";
            }
        });
        return res;
    }
    public String promptLoad(Component gui, String parloc, String typ, String def) {
        JFileChooser fc = createFileChooser(parloc, typ, def);
        if(fc.showOpenDialog(gui) == JFileChooser.APPROVE_OPTION) {
            return fc.getSelectedFile().getAbsolutePath();
        } else {
            return null;
        }
    }
    public String promptSave(Component gui, String parloc, String typ, String def) {
        JFileChooser fc = createFileChooser(parloc, typ, def);
        if(fc.showSaveDialog(gui) == JFileChooser.APPROVE_OPTION) {
            return fc.getSelectedFile().getAbsolutePath();
        } else {
            return null;
        }
    }
}
