/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dk.vajhoej.model.app.AppLookAndFeel;
import dk.vajhoej.model.core.Box;
import dk.vajhoej.model.core.BoxPalette;
import dk.vajhoej.model.core.BoxType;
import dk.vajhoej.model.core.Component;
import dk.vajhoej.model.core.ComponentPalette;
import dk.vajhoej.model.core.ComponentStatus;
import dk.vajhoej.model.core.Connection;
import dk.vajhoej.model.core.ConnectionPalette;
import dk.vajhoej.model.core.ExtModel;
import dk.vajhoej.model.core.Location;
import dk.vajhoej.model.core.Model;
import dk.vajhoej.model.graphic.adv.BoxGraphicWrapper;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;
import dk.vajhoej.model.graphic.config.GraphicConfig;
import dk.vajhoej.model.version.OneNumberVersion;
import dk.vajhoej.model.version.ThreeNumberVersion;
import dk.vajhoej.model.version.TwoNumberVersion;

public class Helper {
	public static List<Location> createLocations() {
		List<Location> res = new ArrayList<Location>();
		res.add(new Location(0, 0, BoxGraphicWrapper.class.getName(), "Web server"));
		res.add(new Location(1, 0, BoxGraphicWrapper.class.getName(), "App server"));
		return res;
	}
	public static Model createModel() {
		Model m = new Model("Test Model", new OneNumberVersion(1));
		Box web = new Box("Web server", new OneNumberVersion(1), "AV", "Custom PC", BoxType.SINGLE_SYS, 6, 8, 1);
		Component webos = new Component("Open Source", "CentOS", new TwoNumberVersion(5,0), ComponentStatus.REMOTE_PASSTHROUGH);
		Component apache = new Component("Open Source", "Apache HTTPD", new ThreeNumberVersion(2,2,11), ComponentStatus.REMOTE_LOADSHARING);
		webos.addComponent(apache);
		web.addComponent(webos);
		m.addBox(web);
		Box app = new Box("App server", new OneNumberVersion(1), "AV", "Custom PC", BoxType.SINGLE_SYS, 6, 8, 1);
		Component appos = new Component("Open Source", "CentOS", new TwoNumberVersion(5,0), ComponentStatus.REMOTE_PASSTHROUGH);
		Component jvm = new Component("SUN", "Java", new TwoNumberVersion(1,6), ComponentStatus.REMOTE_PASSTHROUGH);
		Component jboss = new Component("Open Source", "JBoss AS", new ThreeNumberVersion(4,2,3), ComponentStatus.REMOTE_LOADSHARING);
		Component av = new Component("AV", "TestWebApp", new TwoNumberVersion(1,0), ComponentStatus.REMOTE_SINGLE);
		Component spring = new Component("Open Source", "Spring", new TwoNumberVersion(3,0), ComponentStatus.LOCAL);
		av.addComponent(spring);
		Component hib = new Component("Open Source", "Hibernate", new TwoNumberVersion(3,0), ComponentStatus.LOCAL);
		av.addComponent(hib);
		jboss.addComponent(av);
		jvm.addComponent(jboss);
		appos.addComponent(jvm);
		app.addComponent(appos);
		m.addBox(app);
		return m;
	}
	public static ExtModel createExtModel() {
		return new ExtModel(createModel(), createLocations(), 4, 4);
	}
	public static BoxPalette createBoxPalette() {
		BoxPalette bp = new BoxPalette("Standard boxes", new OneNumberVersion(1));
		bp.addBox(new Box("?", new OneNumberVersion(1), "AV", "Custom PC"));
		bp.addBox(new Box("?", new OneNumberVersion(1), "IBM", "xSeries"));
		bp.addBox(new Box("?", new OneNumberVersion(1), "IBM", "pSeries"));
		bp.addBox(new Box("?", new OneNumberVersion(1), "Dell", "PowerEdge"));
		bp.addBox(new Box("?", new OneNumberVersion(1), "HP", "Proliant"));
		bp.addBox(new Box("?", new OneNumberVersion(1), "SUN", "SPARC"));
        bp.setStore("Box");
		return bp;
	}
	public static List<BoxPalette> createBoxPalettes() {
		List<BoxPalette> res = new ArrayList<BoxPalette>();
		res.add(createBoxPalette());
		return res;
	}
	public static ComponentPalette createComponentPalette() {
		ComponentPalette cp = new ComponentPalette("Standard components", new OneNumberVersion(1));
		cp.addComponent(new Component("Open Source", "CentOS", new TwoNumberVersion(5,0), ComponentStatus.REMOTE_PASSTHROUGH));
		cp.addComponent(new Component("Open Source", "Apache HTTPD", new ThreeNumberVersion(2,2,11), ComponentStatus.REMOTE_LOADSHARING));
		cp.addComponent(new Component("IBM", "Java", new TwoNumberVersion(1,6), ComponentStatus.REMOTE_PASSTHROUGH));
		cp.addComponent(new Component("SUN", "Java", new TwoNumberVersion(1,6), ComponentStatus.REMOTE_PASSTHROUGH));
		cp.addComponent(new Component("Open Source", "JBoss AS", new ThreeNumberVersion(4,2,3), ComponentStatus.REMOTE_LOADSHARING));
		cp.addComponent(new Component("Open Source", "Tomcat", new ThreeNumberVersion(6,0,18), ComponentStatus.REMOTE_LOADSHARING));
		cp.addComponent(new Component("Open Source", "Spring", new TwoNumberVersion(3,0), ComponentStatus.LOCAL));
		cp.addComponent(new Component("AV", "TestWebApp", new TwoNumberVersion(1,0), ComponentStatus.REMOTE_SINGLE));
        cp.setStore("Component");
		return cp;
	}
	public static List<ComponentPalette> createComponentPalettes() {
		List<ComponentPalette> res = new ArrayList<ComponentPalette>();
		res.add(createComponentPalette());
		return res;
	}
	public static ConnectionPalette createConnectionPalette() {
		ConnectionPalette cp = new ConnectionPalette("Standard connections", new OneNumberVersion(1));
		cp.addConnection(new Connection(new OneNumberVersion(1), "HTML/HTTP/TCP/IP"));
		cp.addConnection(new Connection(new OneNumberVersion(1), "SOAP/XML/HTTP/TCP/IP"));
		cp.addConnection(new Connection(new OneNumberVersion(1), "XML/HTTP/TCP/IP"));
		cp.addConnection(new Connection(new OneNumberVersion(1), "XML/FTP/TCP/IP"));
		cp.addConnection(new Connection(new OneNumberVersion(1), "Java/RMI/TCP/IP"));
        cp.setStore("Connection");
		return cp;
	}
	public static List<ConnectionPalette> createConnectionPalettes() {
		List<ConnectionPalette> res = new ArrayList<ConnectionPalette>();
		res.add(createConnectionPalette());
		return res;
	}
	public static Map<String,Boolean> createShow() {
	    Map<String, Boolean> res = new HashMap<String, Boolean>();
        res.put("Box", true);
        res.put("Component", true);
        res.put("Connection", true);
	    return res;
	}
	public static AllGraphicConfig createConfig() {
        GraphicConfig cfgdef = new GraphicConfig(10, 2, new Font("Arial", Font.PLAIN, 12), Color.BLACK, Color.BLACK, Color.LIGHT_GRAY);
        GraphicConfig cfgmod = new GraphicConfig(10, 2, new Font("Arial", Font.PLAIN, 18), Color.BLACK, Color.BLACK, Color.WHITE);
        GraphicConfig cfgpal = new GraphicConfig(10, 2, new Font("Arial", Font.BOLD, 12), Color.BLACK, Color.BLACK, Color.WHITE);
        GraphicConfig cfgav = new GraphicConfig(10, 2, new Font("Arial", Font.PLAIN, 12), Color.BLACK, Color.BLACK, GraphicConfig.PASTEL_RED);
        GraphicConfig cfgibm = new GraphicConfig(10, 2, new Font("Arial", Font.PLAIN, 12), Color.BLACK, Color.BLACK, GraphicConfig.PASTEL_BLUE);
        GraphicConfig cfgsun = new GraphicConfig(10, 2, new Font("Arial", Font.PLAIN, 12), Color.BLACK, Color.BLACK, GraphicConfig.PASTEL_LAVENDER);
        GraphicConfig cfgos = new GraphicConfig(10, 2, new Font("Arial", Font.PLAIN, 12), Color.BLACK, Color.BLACK, GraphicConfig.PASTEL_GREEN);
        GraphicConfig cfgcon = new GraphicConfig(10, 2, new Font("Arial", Font.PLAIN, 12), Color.BLACK, Color.BLACK, GraphicConfig.WHITE_LIGHT_GRAY);
        AllGraphicConfig allcfg = new AllGraphicConfig();
        allcfg.setStandardConfig(cfgdef);
        allcfg.setModelConfig(cfgmod);
        allcfg.setPaletteConfig(cfgpal);
        allcfg.setBoxConfig("AV", cfgav);
        allcfg.setBoxConfig("SUN", cfgsun);
        allcfg.setBoxConfig("IBM", cfgibm);
        allcfg.setComponentConfig("AV", cfgav);
        allcfg.setComponentConfig("SUN", cfgsun);
        allcfg.setComponentConfig("IBM", cfgibm);
        allcfg.setComponentConfig("Open Source", cfgos);
        allcfg.setConnectionConfig(cfgcon);
        return allcfg;
	}
    public static AppLookAndFeel createLaf() {
        return new AppLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel", "Metal", "javax.swing.plaf.metal.DefaultMetalTheme", "Default");
    }
}
