/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import java.util.List;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.core.Box;
import dk.vajhoej.model.core.Component;
import dk.vajhoej.model.core.ComponentStatus;
import dk.vajhoej.model.version.TwoNumberVersion;

public class TestBox {
	@Test
	public void testBasic() {
		Box b = new Box("Some box", new TwoNumberVersion(2,0), "Some vendor", "Foobar");
		b.addComponent(new Component("Some vendor", "Some component", new TwoNumberVersion(1,5), ComponentStatus.REMOTE_SINGLE));
		assertEquals("Box name", "Some box", b.getName());
		assertEquals("Box version", new TwoNumberVersion(2,0), b.getVersion());
		assertEquals("Box vendor", "Some vendor", b.getVendor());
		assertEquals("Box model", "Foobar", b.getModel());
		List<Component> c = b.getComponents();
		assertEquals("Number of components", 1, c.size());
		assertEquals("Component vendor", "Some vendor", c.get(0).getVendor());
		assertEquals("Component name", "Some component", c.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(1,5), c.get(0).getVersion());
		assertEquals("Component status", ComponentStatus.REMOTE_SINGLE, c.get(0).getStatus());
	}
	@Test
	public void testDuplicate() {
		Box b = new Box("Some box", new TwoNumberVersion(2,0), "Some vendor", "Foobar");
		b.addComponent(new Component("Some vendor", "Some component", new TwoNumberVersion(1,5), ComponentStatus.REMOTE_SINGLE));
		Box b2 = b.duplicate();
		assertEquals("Box name", "Some box", b2.getName());
		assertEquals("Box version", new TwoNumberVersion(2,0), b2.getVersion());
		assertEquals("Box vendor", "Some vendor", b2.getVendor());
		assertEquals("Box model", "Foobar", b2.getModel());
		List<Component> c = b2.getComponents();
		assertEquals("Number of components", 1, c.size());
		assertEquals("Component vendor", "Some vendor", c.get(0).getVendor());
		assertEquals("Component name", "Some component", c.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(1,5), c.get(0).getVersion());
		assertEquals("Component status", ComponentStatus.REMOTE_SINGLE, c.get(0).getStatus());
	}
	@Test
	public void testEquals() {
		Box b = new Box("Some box", new TwoNumberVersion(2,0), "Some vendor", "Foobar");
		b.addComponent(new Component("Some vendor", "Some component", new TwoNumberVersion(1,5), ComponentStatus.REMOTE_SINGLE));
		Box b2 = b.duplicate();
		assertTrue("Equals", b.equals(b2));
		assertTrue("Equals", b2.equals(b));
		assertTrue("hash code", b.hashCode() == b2.hashCode());
	}
	@Test
	public void testPersist() {
		Box b = new Box("Some box", new TwoNumberVersion(2,0), "Some vendor", "Foobar");
		b.addComponent(new Component("Some vendor", "Some component", new TwoNumberVersion(1,5), ComponentStatus.REMOTE_SINGLE));
		assertEquals("Type name", "box", b.getType());
		Box b2 = new Box();
		try {
			b2.setAttributes(b.getAttributes());
			b2.setChildren(b.getChildren());
		} catch (ModelException e) {
			fail(e.getMessage());
		}
		assertEquals("Box name", "Some box", b2.getName());
		assertEquals("Box version", new TwoNumberVersion(2,0), b2.getVersion());
		assertEquals("Box vendor", "Some vendor", b2.getVendor());
		assertEquals("Box model", "Foobar", b2.getModel());
		List<Component> c = b2.getComponents();
		assertEquals("Number of components", 1, c.size());
		assertEquals("Component vendor", "Some vendor", c.get(0).getVendor());
		assertEquals("Component name", "Some component", c.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(1,5), c.get(0).getVersion());
		assertEquals("Component status", ComponentStatus.REMOTE_SINGLE, c.get(0).getStatus());
	}
}
