/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import java.util.List;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.core.Component;
import dk.vajhoej.model.core.ComponentStatus;
import dk.vajhoej.model.version.TwoNumberVersion;

public class TestComponent {
	@Test
	public void testBasic() {
		Component c = new Component("Some vendor", "Some component", new TwoNumberVersion(2,0), ComponentStatus.REMOTE_SINGLE);
		c.addComponent(new Component("Some vendor", "Some other component", new TwoNumberVersion(1,5), ComponentStatus.LOCAL));
		assertEquals("Component vendor", "Some vendor", c.getVendor());
		assertEquals("Component name", "Some component", c.getName());
		assertEquals("Component version", new TwoNumberVersion(2,0), c.getVersion());
		assertEquals("Component status", ComponentStatus.REMOTE_SINGLE, c.getStatus());
		List<Component> c2 = c.getComponents();
		assertEquals("Number of components", 1, c2.size());
		assertEquals("Component vendor", "Some vendor", c2.get(0).getVendor());
		assertEquals("Component name", "Some other component", c2.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(1,5), c2.get(0).getVersion());
		assertEquals("Component status", ComponentStatus.LOCAL, c2.get(0).getStatus());
	}
	@Test
	public void testDuplicate() {
		Component c = new Component("Some vendor", "Some component", new TwoNumberVersion(2,0), ComponentStatus.REMOTE_SINGLE);
		c.addComponent(new Component("Some vendor", "Some other component", new TwoNumberVersion(1,5), ComponentStatus.LOCAL));
		Component c2 = c.duplicate();
		assertEquals("Component vendor", "Some vendor", c2.getVendor());
		assertEquals("Component name", "Some component", c2.getName());
		assertEquals("Component version", new TwoNumberVersion(2,0), c2.getVersion());
		assertEquals("Component status", ComponentStatus.REMOTE_SINGLE, c2.getStatus());
		List<Component> c3 = c.getComponents();
		assertEquals("Number of components", 1, c3.size());
		assertEquals("Component vendor", "Some vendor", c3.get(0).getVendor());
		assertEquals("Component name", "Some other component", c3.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(1,5), c3.get(0).getVersion());
		assertEquals("Component status", ComponentStatus.LOCAL, c3.get(0).getStatus());
	}
	@Test
	public void testEquals() {
		Component c = new Component("Some vendor", "Some component", new TwoNumberVersion(2,0), ComponentStatus.REMOTE_SINGLE);
		c.addComponent(new Component("Some vendor", "Some other component", new TwoNumberVersion(1,5), ComponentStatus.LOCAL));
		Component c2 = c.duplicate();
		assertTrue("Equals", c.equals(c2));
		assertTrue("Equals", c2.equals(c));
		assertTrue("hash code", c.hashCode() == c2.hashCode());
	}
	@Test
	public void testPersist() {
		Component c = new Component("Some vendor", "Some component", new TwoNumberVersion(2,0), ComponentStatus.REMOTE_SINGLE);
		c.addComponent(new Component("Some vendor", "Some other component", new TwoNumberVersion(1,5), ComponentStatus.LOCAL));
		assertEquals("Type name", "component", c.getType());
		Component c2 = new Component();
		try {
			c2.setAttributes(c.getAttributes());
			c2.setChildren(c.getChildren());
		} catch (ModelException e) {
			fail(e.getMessage());
		}
		assertEquals("Component vendor", "Some vendor", c2.getVendor());
		assertEquals("Component name", "Some component", c2.getName());
		assertEquals("Component version", new TwoNumberVersion(2,0), c2.getVersion());
		assertEquals("Component status", ComponentStatus.REMOTE_SINGLE, c2.getStatus());
		List<Component> c3 = c.getComponents();
		assertEquals("Number of components", 1, c3.size());
		assertEquals("Component vendor", "Some vendor", c3.get(0).getVendor());
		assertEquals("Component name", "Some other component", c3.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(1,5), c3.get(0).getVersion());
		assertEquals("Component status", ComponentStatus.LOCAL, c3.get(0).getStatus());
	}
}
