/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.core.Connection;
import dk.vajhoej.model.version.TwoNumberVersion;

public class TestConnection {
	@Test
	public void testBasic() {
		Connection c = new Connection(new TwoNumberVersion(2,0), "X/Y/Z");
		assertEquals("Connection stack", "X/Y/Z", c.getStack());
		assertEquals("Connection version", new TwoNumberVersion(2,0), c.getVersion());
	}
	@Test
	public void testDuplicate() {
		Connection c = new Connection(new TwoNumberVersion(2,0), "X/Y/Z");
		Connection c2 = c.duplicate();
		assertEquals("Connection stack", "X/Y/Z", c2.getStack());
		assertEquals("Connection version", new TwoNumberVersion(2,0), c2.getVersion());
	}
	@Test
	public void testEquals() {
		Connection c = new Connection(new TwoNumberVersion(2,0), "X/Y/Z");
		Connection c2 = c.duplicate();
		assertTrue("Equals", c.equals(c2));
		assertTrue("Equals", c2.equals(c));
		assertTrue("hash code", c.hashCode() == c2.hashCode());
	}
	@Test
	public void testPersist() {
		Connection c = new Connection(new TwoNumberVersion(2,0), "X/Y/Z");
		assertEquals("Type name", "connection", c.getType());
		Connection c2 = new Connection();
		try {
			c2.setAttributes(c.getAttributes());
			c2.setChildren(c.getChildren());
		} catch (ModelException e) {
			fail(e.getMessage());
		}
		assertEquals("Connection stack", "X/Y/Z", c2.getStack());
		assertEquals("Connection version", new TwoNumberVersion(2,0), c2.getVersion());
	}
}
