/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import java.util.List;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.core.Box;
import dk.vajhoej.model.core.BoxPalette;
import dk.vajhoej.model.core.Component;
import dk.vajhoej.model.core.ComponentPalette;
import dk.vajhoej.model.core.Connection;
import dk.vajhoej.model.core.ConnectionPalette;
import dk.vajhoej.model.core.ExtModel;
import dk.vajhoej.model.core.Location;
import dk.vajhoej.model.core.Model;
import dk.vajhoej.model.graphic.adv.BoxGraphicWrapper;
import dk.vajhoej.model.version.OneNumberVersion;
import dk.vajhoej.model.version.ThreeNumberVersion;
import dk.vajhoej.model.version.TwoNumberVersion;

public class TestCore {
	@Test
	public void testBoxPalette() {
		BoxPalette bp = Helper.createBoxPalette();
		List<Box> b = bp.getBoxes();
		assertEquals("Number of boxes", 6, b.size());
		assertEquals("Box version", new OneNumberVersion(1), b.get(0).getVersion());
		assertEquals("Box name", "?", b.get(0).getName());
		assertEquals("Box vendor", "AV", b.get(0).getVendor());
		assertEquals("Box model", "Custom PC", b.get(0).getModel());
		assertEquals("Box version", new OneNumberVersion(1), b.get(1).getVersion());
		assertEquals("Box name", "?", b.get(1).getName());
		assertEquals("Box vendor", "IBM", b.get(1).getVendor());
		assertEquals("Box model", "xSeries", b.get(1).getModel());
		assertEquals("Box version", new OneNumberVersion(1), b.get(2).getVersion());
		assertEquals("Box name", "?", b.get(2).getName());
		assertEquals("Box vendor", "IBM", b.get(2).getVendor());
		assertEquals("Box model", "pSeries", b.get(2).getModel());
		assertEquals("Box version", new OneNumberVersion(1), b.get(3).getVersion());
		assertEquals("Box name", "?", b.get(3).getName());
		assertEquals("Box vendor", "Dell", b.get(3).getVendor());
		assertEquals("Box model", "PowerEdge", b.get(3).getModel());
		assertEquals("Box version", new OneNumberVersion(1), b.get(4).getVersion());
		assertEquals("Box name", "?", b.get(4).getName());
		assertEquals("Box vendor", "HP", b.get(4).getVendor());
		assertEquals("Box model", "Proliant", b.get(4).getModel());
		assertEquals("Box version", new OneNumberVersion(1), b.get(5).getVersion());
		assertEquals("Box name", "?", b.get(5).getName());
		assertEquals("Box vendor", "SUN", b.get(5).getVendor());
		assertEquals("Box model", "SPARC", b.get(5).getModel());
	}
	@Test
	public void testComponentPalette() {
		ComponentPalette cp = Helper.createComponentPalette();
		List<Component> c = cp.getComponents();
		assertEquals("Number of components", 8, c.size());
		assertEquals("Component vendor", "Open Source", c.get(0).getVendor());
		assertEquals("Component name", "CentOS", c.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(5,0), c.get(0).getVersion());
		assertEquals("Component vendor", "Open Source", c.get(1).getVendor());
		assertEquals("Component name", "Apache HTTPD", c.get(1).getName());
		assertEquals("Component version", new ThreeNumberVersion(2,2,11), c.get(1).getVersion());
		assertEquals("Component vendor", "IBM", c.get(2).getVendor());
		assertEquals("Component name", "Java", c.get(2).getName());
		assertEquals("Component version", new TwoNumberVersion(1,6), c.get(2).getVersion());
		assertEquals("Component vendor", "SUN", c.get(3).getVendor());
		assertEquals("Component name", "Java", c.get(3).getName());
		assertEquals("Component version", new TwoNumberVersion(1,6), c.get(3).getVersion());
		assertEquals("Component vendor", "Open Source", c.get(4).getVendor());
		assertEquals("Component name", "JBoss AS", c.get(4).getName());
		assertEquals("Component version", new ThreeNumberVersion(4,2,3), c.get(4).getVersion());
		assertEquals("Component vendor", "Open Source", c.get(5).getVendor());
		assertEquals("Component name", "Tomcat", c.get(5).getName());
		assertEquals("Component version", new ThreeNumberVersion(6,0,18), c.get(5).getVersion());
		assertEquals("Component vendor", "Open Source", c.get(6).getVendor());
		assertEquals("Component name", "Spring", c.get(6).getName());
		assertEquals("Component version", new TwoNumberVersion(3,0), c.get(6).getVersion());
		assertEquals("Component vendor", "AV", c.get(7).getVendor());
		assertEquals("Component name", "TestWebApp", c.get(7).getName());
		assertEquals("Component version", new TwoNumberVersion(1,0), c.get(7).getVersion());
	}
	@Test
	public void testConnectionPalette() {
		ConnectionPalette cp = Helper.createConnectionPalette();
		List<Connection> c = cp.getConnections();
		assertEquals("Number of connections", 5, c.size());
		assertEquals("Connection stack", "HTML/HTTP/TCP/IP", c.get(0).getStack());
		assertEquals("Connection stack", "SOAP/XML/HTTP/TCP/IP", c.get(1).getStack());
		assertEquals("Connection stack", "XML/HTTP/TCP/IP", c.get(2).getStack());
		assertEquals("Connection stack", "XML/FTP/TCP/IP", c.get(3).getStack());
		assertEquals("Connection stack", "Java/RMI/TCP/IP", c.get(4).getStack());
	}
	@Test
	public void testModel() {
		realTestModel(Helper.createModel());
	}
	@Test
	public void testLocations() {
		realTestLocations(Helper.createLocations());
	}
	@Test
	public void testExtModel() {
		ExtModel em = Helper.createExtModel();
		realTestModel(em.getReal());
		realTestLocations(em.getLocations());
	}
	private void realTestModel(Model m) {
		assertEquals("Model version", new OneNumberVersion(1), m.getVersion());
		List<Box> b = m.getBoxes();
		assertEquals("Number of boxes", 2, b.size());
		assertEquals("Box version", new OneNumberVersion(1), b.get(0).getVersion());
		assertEquals("Box name", "Web server", b.get(0).getName());
		assertEquals("Box vendor", "AV", b.get(0).getVendor());
		assertEquals("Box model", "Custom PC", b.get(0).getModel());
		assertEquals("Box version", new OneNumberVersion(1), b.get(1).getVersion());
		assertEquals("Box name", "App server", b.get(1).getName());
		assertEquals("Box vendor", "AV", b.get(1).getVendor());
		assertEquals("Box model", "Custom PC", b.get(1).getModel());
		List<Component> c1 = b.get(0).getComponents();
		assertEquals("Number of components", 1, c1.size());
		assertEquals("Component vendor", "Open Source", c1.get(0).getVendor());
		assertEquals("Component name", "CentOS", c1.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(5,0), c1.get(0).getVersion());
		c1 = c1.get(0).getComponents();
		assertEquals("Number of components", 1, c1.size());
		assertEquals("Component vendor", "Open Source", c1.get(0).getVendor());
		assertEquals("Component name", "Apache HTTPD", c1.get(0).getName());
		assertEquals("Component version", new ThreeNumberVersion(2,2,11), c1.get(0).getVersion());
		List<Component> c2 = b.get(1).getComponents();
		assertEquals("Number of components", 1, c2.size());
		assertEquals("Component vendor", "Open Source", c1.get(0).getVendor());
		assertEquals("Component name", "CentOS", c2.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(5,0), c2.get(0).getVersion());
		c2 = c2.get(0).getComponents();
		assertEquals("Number of components", 1, c2.size());
		assertEquals("Component vendor", "SUN", c2.get(0).getVendor());
		assertEquals("Component name", "Java", c2.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(1,6), c2.get(0).getVersion());
		c2 = c2.get(0).getComponents();
		assertEquals("Number of components", 1, c2.size());
		assertEquals("Component vendor", "Open Source", c2.get(0).getVendor());
		assertEquals("Component name", "JBoss AS", c2.get(0).getName());
		assertEquals("Component version", new ThreeNumberVersion(4,2,3), c2.get(0).getVersion());
		c2 = c2.get(0).getComponents();
		assertEquals("Number of components", 1, c2.size());
		assertEquals("Component vendor", "AV", c2.get(0).getVendor());
		assertEquals("Component name", "TestWebApp", c2.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(1,0), c2.get(0).getVersion());
		c2 = c2.get(0).getComponents();
		assertEquals("Number of components", 2, c2.size());
		assertEquals("Component vendor", "Open Source", c2.get(0).getVendor());
		assertEquals("Component name", "Spring", c2.get(0).getName());
		assertEquals("Component version", new TwoNumberVersion(3,0), c2.get(0).getVersion());
		assertEquals("Component vendor", "Open Source", c2.get(1).getVendor());
		assertEquals("Component name", "Hibernate", c2.get(1).getName());
		assertEquals("Component version", new TwoNumberVersion(3,0), c2.get(1).getVersion());
	}
	private void realTestLocations(List<Location> loc) {
		assertEquals("Number of locations", 2, loc.size());
		assertEquals("Location x", 0, loc.get(0).getX());
		assertEquals("Location y", 0, loc.get(0).getY());
		assertEquals("Location class", BoxGraphicWrapper.class.getName(), loc.get(0).getClazz());
		assertEquals("Location id", "Web server", loc.get(0).getId());
		assertEquals("Location x", 1, loc.get(1).getX());
		assertEquals("Location y", 0, loc.get(1).getY());
		assertEquals("Location class", BoxGraphicWrapper.class.getName(), loc.get(1).getClazz());
		assertEquals("Location id", "App server", loc.get(1).getId());
	}
}
