/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.core.Location;

public class TestLocation {
	@Test
	public void testBasic() {
		Location loc = new Location(2, 3, "Foo", "Bar");
		assertEquals("Location x", 2, loc.getX());
		assertEquals("Location y", 3, loc.getY());
		assertEquals("Location clazz", "Foo", loc.getClazz());
		assertEquals("Location id", "Bar", loc.getId());
	}
	@Test
	public void testClone() {
		Location loc = new Location(2, 3, "Foo", "Bar");
		Location loc2 = loc.clone();
		assertEquals("Location x", 2, loc2.getX());
		assertEquals("Location y", 3, loc.getY());
		assertEquals("Location clazz", "Foo", loc2.getClazz());
		assertEquals("Location id", "Bar", loc2.getId());
	}
	@Test
	public void testEquals() {
		Location loc = new Location(2, 3, "Foo", "Bar");
		Location loc2 = loc.clone();
		assertTrue("Equals", loc.equals(loc2));
		assertTrue("Equals", loc2.equals(loc));
		assertTrue("hash code", loc.hashCode() == loc2.hashCode());
	}
	@Test
	public void testPersist() {
		Location loc = new Location(2, 3, "Foo", "Bar");
		assertEquals("Type name", "location", loc.getType());
		Location loc2 = new Location();
		try {
			loc2.setAttributes(loc.getAttributes());
			loc2.setChildren(loc.getChildren());
		} catch (ModelException e) {
			fail(e.getMessage());
		}
		assertEquals("Location x", 2, loc2.getX());
		assertEquals("Location y", 3, loc.getY());
		assertEquals("Location clazz", "Foo", loc2.getClazz());
		assertEquals("Location id", "Bar", loc2.getId());
	}
}
