/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.version.NoVersion;
import dk.vajhoej.model.version.NoVersionFactory;

public class TestNoVersion {
	@Test
	public void testBasic() {
		NoVersion ver = new NoVersion();
        assertEquals("Type", NoVersion.class, ver.getClass());
	}
	@Test
	public void testBump() {
		NoVersion ver = new NoVersion();
        assertEquals("Type", NoVersion.class, ver.getClass());
		try {
			ver = ver.bumpLeast();
			fail("Unpexpected success of bump least");
		} catch(ModelRuntimeException ex) {
			// expected
		}
		try {
			ver = ver.bumpMost();
			fail("Unpexpected success of bump most");
		} catch(ModelRuntimeException ex) {
			// expected
		}
		try {
			ver = ver.bumpSecondLeast();
			fail("Unpexpected success of bump second least");
		} catch(ModelRuntimeException ex) {
			// expected
		}
		try {
			ver = ver.bumpSecondMost();
			fail("Unpexpected success of bump second most");
		} catch(ModelRuntimeException ex) {
			// expected
		}
	}
	@Test
	public void testFactory() {
		NoVersionFactory fact = new NoVersionFactory();
		NoVersion ver = fact.create("");
        assertEquals("Type", NoVersion.class, ver.getClass());
	}
	@Test
	public void testIndirect() {
		try {
			NoVersion ver = (NoVersion)Version.parse("");
	        assertEquals("Type", NoVersion.class, ver.getClass());
		} catch (ModelException e) {
			fail(e.getMessage());
		}
	}
}
