/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.version.OneNumberVersion;
import dk.vajhoej.model.version.OneNumberVersionFactory;

public class TestOneNumberVersion {
	@Test
	public void testBasic() {
		OneNumberVersion ver = new OneNumberVersion(1);
        assertEquals("Type", OneNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
	}
	@Test
	public void testBump() {
		OneNumberVersion ver = new OneNumberVersion(1);
        assertEquals("Type", OneNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		ver = ver.bumpLeast();
        assertEquals("Type", OneNumberVersion.class, ver.getClass());
		assertEquals("Major", 2, ver.getMajor());
		ver = ver.bumpSecondLeast();
        assertEquals("Type", OneNumberVersion.class, ver.getClass());
		assertEquals("Major", 3, ver.getMajor());
		ver = ver.bumpSecondMost();
        assertEquals("Type", OneNumberVersion.class, ver.getClass());
		assertEquals("Major", 4, ver.getMajor());
		ver = ver.bumpMost();
        assertEquals("Type", OneNumberVersion.class, ver.getClass());
		assertEquals("Major", 5, ver.getMajor());
	}
	@Test
	public void testFactory() {
		OneNumberVersionFactory fact = new OneNumberVersionFactory();
		OneNumberVersion ver = fact.create("1");
        assertEquals("Type", OneNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
	}
	@Test
	public void testIndirect() {
		try {
			OneNumberVersion ver = (OneNumberVersion)Version.parse("1");
	        assertEquals("Type", OneNumberVersion.class, ver.getClass());
			assertEquals("Major", 1, ver.getMajor());
		} catch (ModelException e) {
			fail(e.getMessage());
		}
	}
}
