/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.version.ThreeNumberVersion;
import dk.vajhoej.model.version.ThreeNumberVersionFactory;

public class TestThreeNumberVersion {
	@Test
	public void testBasic() {
		ThreeNumberVersion ver = new ThreeNumberVersion(1,2,3);
        assertEquals("Type", ThreeNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 2, ver.getMinor());
		assertEquals("Update", 3, ver.getUpdate());
	}
	@Test
	public void testBump() {
		ThreeNumberVersion ver = new ThreeNumberVersion(1,2,3);
        assertEquals("Type", ThreeNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 2, ver.getMinor());
		assertEquals("Update", 3, ver.getUpdate());
		ver = ver.bumpLeast();
        assertEquals("Type", ThreeNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 2, ver.getMinor());
		assertEquals("Update", 4, ver.getUpdate());
		ver = ver.bumpSecondLeast();
        assertEquals("Type", ThreeNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 3, ver.getMinor());
		assertEquals("Update", 0, ver.getUpdate());
		ver = ver.bumpSecondMost();
        assertEquals("Type", ThreeNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 4, ver.getMinor());
		assertEquals("Update", 0, ver.getUpdate());
		ver = ver.bumpMost();
        assertEquals("Type", ThreeNumberVersion.class, ver.getClass());
		assertEquals("Major", 2, ver.getMajor());
		assertEquals("Minor", 0, ver.getMinor());
		assertEquals("Update", 0, ver.getUpdate());
	}
	@Test
	public void testFactory() {
		ThreeNumberVersionFactory fact = new ThreeNumberVersionFactory();
		ThreeNumberVersion ver = fact.create("1.2.3");
        assertEquals("Type", ThreeNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 2, ver.getMinor());
		assertEquals("Update", 3, ver.getUpdate());
	}
	@Test
	public void testIndirect() {
		try {
			ThreeNumberVersion ver = (ThreeNumberVersion)Version.parse("1.2.3");
	        assertEquals("Type", ThreeNumberVersion.class, ver.getClass());
			assertEquals("Major", 1, ver.getMajor());
			assertEquals("Minor", 2, ver.getMinor());
			assertEquals("Update", 3, ver.getUpdate());
		} catch (ModelException e) {
			fail(e.getMessage());
		}
	}
}
