/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.version.TwoNumberVersion;
import dk.vajhoej.model.version.TwoNumberVersionFactory;

public class TestTwoNumberVersion {
	@Test
	public void testBasic() {
		TwoNumberVersion ver = new TwoNumberVersion(1,2);
        assertEquals("Type", TwoNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 2, ver.getMinor());
	}
	@Test
	public void testBump() {
		TwoNumberVersion ver = new TwoNumberVersion(1,2);
        assertEquals("Type", TwoNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 2, ver.getMinor());
		ver = ver.bumpLeast();
        assertEquals("Type", TwoNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 3, ver.getMinor());
		ver = ver.bumpSecondLeast();
        assertEquals("Type", TwoNumberVersion.class, ver.getClass());
		assertEquals("Major", 2, ver.getMajor());
		assertEquals("Minor", 0, ver.getMinor());
		ver = ver.bumpSecondMost();
        assertEquals("Type", TwoNumberVersion.class, ver.getClass());
		assertEquals("Major", 2, ver.getMajor());
		assertEquals("Minor", 1, ver.getMinor());
		ver = ver.bumpMost();
        assertEquals("Type", TwoNumberVersion.class, ver.getClass());
		assertEquals("Major", 3, ver.getMajor());
		assertEquals("Minor", 0, ver.getMinor());
	}
	@Test
	public void testFactory() {
		TwoNumberVersionFactory fact = new TwoNumberVersionFactory();
		TwoNumberVersion ver = fact.create("1.2");
        assertEquals("Type", TwoNumberVersion.class, ver.getClass());
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 2, ver.getMinor());
	}
	@Test
	public void testIndirect() {
		try {
			TwoNumberVersion ver = (TwoNumberVersion)Version.parse("1.2");
	        assertEquals("Type", TwoNumberVersion.class, ver.getClass());
			assertEquals("Major", 1, ver.getMajor());
			assertEquals("Minor", 2, ver.getMinor());
		} catch (ModelException e) {
			fail(e.getMessage());
		}
	}
}
