/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

import org.junit.Test;
import static org.junit.Assert.*;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.Persister;
import dk.vajhoej.model.core.Model;
import dk.vajhoej.model.persist.XmlPersister;

public class TestXmlPersister {
    private static String loadFile(String fnm) throws IOException {
        StringBuffer sb = new StringBuffer("");
        BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(fnm)));
        String line;
        while((line = in.readLine()) != null) {
            sb.append(line);
            sb.append(System.getProperty("line.separator"));
        }
        in.close();
        return sb.toString();
    }
    private static final String F1 = "m1.xml";
    private static final String F2 = "m2.xml";
	@Test
	public void testSaveLoad() {
		try {
			Model m = Helper.createModel();
			Persister p = new XmlPersister();
			p.save(m, F1);
			Model m2 = (Model)p.load(F1, Model.class);
			p.save(m2, F2);
			assertEquals("File content", loadFile(F1), loadFile(F2));
			(new File(F1)).delete();
			(new File(F2)).delete();
		} catch (ModelException e) {
			fail(e.getMessage());
		} catch (IOException e) {
			fail(e.getMessage());
		}
	}
}
