/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.version;

import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.Version;

/**
 * Class OneNumberVersion represent version numbers in the 
 * format n.
 */
public class OneNumberVersion extends Version {
	private static final long serialVersionUID = 1L;
	private int major;
    /**
     * Create instance of OneNumberVersion.
     * @param major major version number
     */
	public OneNumberVersion(int major) {
		this.major = major;
	}
    /**
     * Get major version number.
     * @return major version
     */
	public int getMajor() {
		return major;
	}
	@Override
	public int compareTo(Version v) {
		if(v instanceof OneNumberVersion) {
			OneNumberVersion v2 = (OneNumberVersion)v;
			return major - v2.major;
		} else {
			throw new ModelRuntimeException("Comparing incompatible version numbers");
		}
	}
	@Override
	public String toString() {
		return Integer.toString(major);
	}
	@Override
	public OneNumberVersion bumpLeast() {
		return new OneNumberVersion(major+1);
	}
	@Override
	public OneNumberVersion bumpMost() {
		return bumpLeast();
	}
	@Override
	public OneNumberVersion bumpSecondLeast() {
		return bumpMost();
	}
	@Override
	public OneNumberVersion bumpSecondMost() {
		return bumpLeast();
	}
}
