/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.version;

import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.Version;

/**
 * Class StringVersion represent version numbers that
 * are non numeric. 
 */
public class StringVersion extends Version {
	private static final long serialVersionUID = 1L;
	private String version;
	/**
	 * Create instance of StringVersion.
	 * @param version version
	 */
	public StringVersion(String version) {
		this.version = version;
	}
	/**
	 * Get version.
	 * @return version
	 */
	public String getVersion() {
		return version;
	}
	@Override
	public int compareTo(Version v) {
		throw new ModelRuntimeException("Comparing string version number");
	}
	@Override
	public String toString() {
		return version;
	}
	@Override
	public StringVersion bumpLeast() {
		throw new ModelRuntimeException("Can not bump string version number");
	}
	@Override
	public StringVersion bumpMost() {
		throw new ModelRuntimeException("Can not bump string version number");
	}
	@Override
	public StringVersion bumpSecondLeast() {
		throw new ModelRuntimeException("Can not bump string version number");
	}
	@Override
	public StringVersion bumpSecondMost() {
		throw new ModelRuntimeException("Can not bump string version number");
	}
}
