/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.version;

import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.Version;

/**
 * Class ThreeNumberVersion represent version numbers in the 
 * format n.n.n.
 */
public class ThreeNumberVersion extends Version {
	private static final long serialVersionUID = 1L;
	private int major;
	private int minor;
	private int update;
    /**
     * Create instance of ThreeNumberVersion.
     * @param major major version number
     * @param minor minor version number
     * @param update update number
     */
	public ThreeNumberVersion(int major, int minor, int update) {
		this.major = major;
		this.minor = minor;
		this.update = update;
	}
    /**
     * Get major version number.
     * @return major version
     */
    public int getMajor() {
        return major;
    }
    /**
     * Get minor version number.
     * @return minor version
     */
    public int getMinor() {
        return minor;
    }
    /**
     * Get update number.
     * @return update
     */
    public int getUpdate() {
        return update;
    }
	@Override
	public int compareTo(Version v) {
		if(v instanceof ThreeNumberVersion) {
			ThreeNumberVersion v2 = (ThreeNumberVersion)v;
			if(major == v2.major) {
				if(minor == v2.minor){
					return update - v2.update;
				} else {
					return minor - v2.minor;
				}
			} else {
				return major - v2.major;
			}
		} else {
			throw new ModelRuntimeException("Comparing incompatible version numbers");
		}
	}
	@Override
	public String toString() {
		return major + "." + minor + "." + update;
	}
	@Override
	public ThreeNumberVersion bumpLeast() {
		return new ThreeNumberVersion(major, minor, update+1);
	}
	@Override
	public ThreeNumberVersion bumpMost() {
		return new ThreeNumberVersion(major+1, 0, 0);
	}
	@Override
	public ThreeNumberVersion bumpSecondLeast() {
		return new ThreeNumberVersion(major, minor+1, 0);
	}
	@Override
	public ThreeNumberVersion bumpSecondMost() {
		return new ThreeNumberVersion(major, minor+1, 0);
	}
}
