/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.version;

import dk.vajhoej.model.Version;
import dk.vajhoej.model.VersionFactory;

/**
 * Class ThreeNumberVersionFactory return instances of ThreerNumberVersion.
 */
public class ThreeNumberVersionFactory implements VersionFactory {
	static {
		Version.registerFactory(new ThreeNumberVersionFactory());
	}
	public ThreeNumberVersion create(String s) {
		String[] parts = s.split("\\.");
		if(parts.length == 3) {
            try {
                return new ThreeNumberVersion(Integer.parseInt(parts[0]), Integer.parseInt(parts[1]), Integer.parseInt(parts[2]));
            } catch(NumberFormatException ex) {
                return null;
            }
		} else {
			return null;
		}
	}
}
