/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.version;

import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.Version;

/**
 * ClassTwoNumberVersion represent version numbers in the 
 * format n.n.
 */
public class TwoNumberVersion extends Version {
	private static final long serialVersionUID = 1L;
	private int major;
	private int minor;
    /**
     * Create instance of FourNumberVersion.
     * @param major major version number
     * @param minor minor version number
     */
	public TwoNumberVersion(int major, int minor) {
		this.major = major;
		this.minor = minor;
	}
    /**
     * Get major version number.
     * @return major version
     */
    public int getMajor() {
        return major;
    }
    /**
     * Get minor version number.
     * @return minor version
     */
    public int getMinor() {
        return minor;
    }
	@Override
	public int compareTo(Version v) {
		if(v instanceof TwoNumberVersion) {
			TwoNumberVersion v2 = (TwoNumberVersion)v;
			if(major == v2.major) {
				return minor - v2.minor;
			} else {
				return major - v2.major;
			}
		} else {
			throw new ModelRuntimeException("Comparing incompatible version numbers");
		}
	}
	@Override
	public String toString() {
		return major + "." + minor;
	}
	@Override
	public TwoNumberVersion bumpLeast() {
		return new TwoNumberVersion(major, minor+1);
	}
	@Override
	public TwoNumberVersion bumpMost() {
		return new TwoNumberVersion(major+1, 0);
	}
	@Override
	public TwoNumberVersion bumpSecondLeast() {
		return bumpMost();
	}
	@Override
	public TwoNumberVersion bumpSecondMost() {
		return bumpLeast();
	}
}
